@@ -196,7 +196,7 @@ def find_resource_pool_by_cluster(content, resource_pool_name='Resources', clust
 
 
 def find_network_by_name(content, network_name, datacenter_name=None):
-    return find_object_by_name(content, quote_obj_name(network_name), [vim.Network], datacenter_name)
+    return find_object_by_name(content, network_name, [vim.Network], datacenter_name)
 
 
 def find_vm_by_id(content, vm_id, vm_id_type="vm_name", datacenter=None,
@@ -317,6 +317,7 @@ def gather_vm_facts(content, vm):
         'vnc': {},
         'moid': vm._moId,
         'vimref': "vim.VirtualMachine:%s" % vm._moId,
+        'advanced_settings': {},
     }
 
     # facts that may or may not exist
@@ -372,6 +373,10 @@ def gather_vm_facts(content, vm):
 
         facts['customvalues'][kn] = value_obj.value
 
+    # Resolve advanced settings
+    for advanced_setting in vm.config.extraConfig:
+        facts['advanced_settings'][advanced_setting.key] = advanced_setting.value
+
     net_dict = {}
     vmnet = _get_vm_prop(vm, ('guest', 'net'))
     if vmnet:
@@ -1413,19 +1418,20 @@ class PyVmomi(object):
         return find_folder_by_name(self.content, folder_name=folder_name)
 
     # Datastore cluster
-    def find_datastore_cluster_by_name(self, datastore_cluster_name, datacenter=None):
+    def find_datastore_cluster_by_name(self, datastore_cluster_name, datacenter=None, folder=None):
         """
         Get datastore cluster managed object by name
         Args:
             datastore_cluster_name: Name of datastore cluster
             datacenter: Managed object of the datacenter
+            folder: Managed object of the folder which holds datastore
 
         Returns: Datastore cluster managed object if found else None
 
         """
         if datacenter and hasattr(datacenter, 'datastoreFolder'):
             folder = datacenter.datastoreFolder
-        else:
+        if not folder:
             folder = self.content.rootFolder
 
         data_store_clusters = get_all_objs(self.content, [vim.StoragePod], folder=folder)
@@ -1434,6 +1440,48 @@ class PyVmomi(object):
                 return dsc
         return None
 
+    def get_recommended_datastore(self, datastore_cluster_obj=None):
+        """
+        Return Storage DRS recommended datastore from datastore cluster
+        Args:
+            datastore_cluster_obj: datastore cluster managed object
+
+        Returns: Name of recommended datastore from the given datastore cluster
+
+        """
+        if datastore_cluster_obj is None:
+            return None
+        # Check if Datastore Cluster provided by user is SDRS ready
+        sdrs_status = datastore_cluster_obj.podStorageDrsEntry.storageDrsConfig.podConfig.enabled
+        if sdrs_status:
+            # We can get storage recommendation only if SDRS is enabled on given datastorage cluster
+            pod_sel_spec = vim.storageDrs.PodSelectionSpec()
+            pod_sel_spec.storagePod = datastore_cluster_obj
+            storage_spec = vim.storageDrs.StoragePlacementSpec()
+            storage_spec.podSelectionSpec = pod_sel_spec
+            storage_spec.type = 'create'
+
+            try:
+                rec = self.content.storageResourceManager.RecommendDatastores(storageSpec=storage_spec)
+                rec_action = rec.recommendations[0].action[0]
+                return rec_action.destination.name
+            except Exception:
+                # There is some error so we fall back to general workflow
+                pass
+        datastore = None
+        datastore_freespace = 0
+        for ds in datastore_cluster_obj.childEntity:
+            if isinstance(ds, vim.Datastore) and ds.summary.freeSpace > datastore_freespace:
+                # If datastore field is provided, filter destination datastores
+                if not self.is_datastore_valid(datastore_obj=ds):
+                    continue
+
+                datastore = ds
+                datastore_freespace = ds.summary.freeSpace
+        if datastore:
+            return datastore.name
+        return None
+
     # Resource pool
     def find_resource_pool_by_name(self, resource_pool_name, folder=None):
         """
@@ -1558,6 +1606,30 @@ class PyVmomi(object):
 
         self.module.fail_json(msg="No vmdk file found for path specified [%s]" % vmdk_path)
 
+    def find_first_class_disk_by_name(self, disk_name, datastore_obj):
+        """
+        Get first-class disk managed object by name
+        Args:
+            disk_name: Name of the first-class disk
+            datastore_obj: Managed object of datastore
+
+        Returns: First-class disk managed object if found else None
+
+        """
+
+        if self.is_vcenter():
+            for id in self.content.vStorageObjectManager.ListVStorageObject(datastore_obj):
+                disk = self.content.vStorageObjectManager.RetrieveVStorageObject(id, datastore_obj)
+                if disk.config.name == disk_name:
+                    return disk
+        else:
+            for id in self.content.vStorageObjectManager.HostListVStorageObject(datastore_obj):
+                disk = self.content.vStorageObjectManager.HostRetrieveVStorageObject(id, datastore_obj)
+                if disk.config.name == disk_name:
+                    return disk
+
+        return None
+
     #
     # Conversion to JSON
     #
