@@ -9,7 +9,7 @@ from __future__ import absolute_import, division, print_function
 __metaclass__ = type
 
 
-DOCUMENTATION = '''
+DOCUMENTATION = r'''
 ---
 module: vmware_dvs_portgroup
 short_description: Create or remove a Distributed vSwitch portgroup.
@@ -218,7 +218,7 @@ extends_documentation_fragment:
 
 '''
 
-EXAMPLES = '''
+EXAMPLES = r'''
 - name: Create vlan portgroup
   community.vmware.vmware_dvs_portgroup:
     hostname: '{{ vcenter_hostname }}'
@@ -285,21 +285,21 @@ EXAMPLES = '''
     portgroup_type: earlyBinding
     state: present
     network_policy:
-      promiscuous: yes
-      forged_transmits: yes
-      mac_changes: yes
+      promiscuous: true
+      forged_transmits: true
+      mac_changes: true
     port_policy:
-      block_override: yes
-      ipfix_override: yes
-      live_port_move: yes
-      network_rp_override: yes
-      port_config_reset_at_disconnect: yes
-      security_override: yes
-      shaping_override: yes
-      traffic_filter_override: yes
-      uplink_teaming_override: yes
-      vendor_config_override: yes
-      vlan_override: yes
+      block_override: true
+      ipfix_override: true
+      live_port_move: true
+      network_rp_override: true
+      port_config_reset_at_disconnect: true
+      security_override: true
+      shaping_override: true
+      traffic_filter_override: true
+      uplink_teaming_override: true
+      vendor_config_override: true
+      vlan_override: true
   delegate_to: localhost
 '''
 
@@ -500,7 +500,7 @@ class VMwareDvsPortgroup(PyVmomi):
         if self.module.params['vlan_trunk']:
             if not isinstance(defaultPortConfig.vlan, vim.dvs.VmwareDistributedVirtualSwitch.TrunkVlanSpec):
                 return 'update'
-            if map(lambda x: (x.start, x.end), defaultPortConfig.vlan.vlanId) != self.create_vlan_list():
+            if list(map(lambda x: (x.start, x.end), defaultPortConfig.vlan.vlanId)) != self.create_vlan_list():
                 return 'update'
         elif self.module.params['vlan_private']:
             if not isinstance(defaultPortConfig.vlan, vim.dvs.VmwareDistributedVirtualSwitch.PvlanSpec):
