@@ -50,6 +50,7 @@ DOCUMENTATION = r'''
         - Whether or not to force push or pull an image.
         - When building, force the build even if the image already exists.
       type: bool
+      default: False
     state:
       description:
         - Whether an image should be present, absent, or built.
@@ -62,7 +63,6 @@ DOCUMENTATION = r'''
     validate_certs:
       description:
         - Require HTTPS and validate certificates when pulling or pushing. Also used during build if a pull or push is necessary.
-      default: True
       type: bool
       aliases:
         - tlsverify
@@ -165,26 +165,26 @@ DOCUMENTATION = r'''
 
 EXAMPLES = r"""
 - name: Pull an image
-  container.podman.podman_image:
+  containers.podman.podman_image:
     name: quay.io/bitnami/wildfly
 
 - name: Remove an image
-  container.podman.podman_image:
+  containers.podman.podman_image:
     name: quay.io/bitnami/wildfly
     state: absent
 
 - name: Pull a specific version of an image
-  container.podman.podman_image:
+  containers.podman.podman_image:
     name: redis
     tag: 4
 
 - name: Build a basic OCI image
-  container.podman.podman_image:
+  containers.podman.podman_image:
     name: nginx
     path: /path/to/build/dir
 
 - name: Build a basic OCI image with advanced parameters
-  container.podman.podman_image:
+  containers.podman.podman_image:
     name: nginx
     path: /path/to/build/dir
     build:
@@ -197,14 +197,14 @@ EXAMPLES = r"""
         info: Load balancer for my cool app
 
 - name: Build a Docker formatted image
-  container.podman.podman_image:
+  containers.podman.podman_image:
     name: nginx
     path: /path/to/build/dir
     build:
       format: docker
 
 - name: Build and push an image using existing credentials
-  container.podman.podman_image:
+  containers.podman.podman_image:
     name: nginx
     path: /path/to/build/dir
     push: yes
@@ -212,7 +212,7 @@ EXAMPLES = r"""
       dest: quay.io/acme
 
 - name: Build and push an image using an auth file
-  container.podman.podman_image:
+  containers.podman.podman_image:
     name: nginx
     push: yes
     auth_file: /etc/containers/auth.json
@@ -220,7 +220,7 @@ EXAMPLES = r"""
       dest: quay.io/acme
 
 - name: Build and push an image using username and password
-  container.podman.podman_image:
+  containers.podman.podman_image:
     name: nginx
     push: yes
     username: bugs
@@ -229,7 +229,7 @@ EXAMPLES = r"""
       dest: quay.io/acme
 
 - name: Build and push an image to multiple registries
-  container.podman.podman_image:
+  containers.podman.podman_image:
     name: "{{ item }}"
     path: /path/to/build/dir
     push: yes
@@ -239,7 +239,7 @@ EXAMPLES = r"""
     - docker.io/acme/nginx
 
 - name: Build and push an image to multiple registries with separate parameters
-  container.podman.podman_image:
+  containers.podman.podman_image:
     name: "{{ item.name }}"
     tag: "{{ item.tag }}"
     path: /path/to/build/dir
@@ -382,6 +382,7 @@ import json
 import re
 import shlex
 
+from ansible.module_utils._text import to_native
 from ansible.module_utils.basic import AnsibleModule
 from ansible_collections.containers.podman.plugins.module_utils.podman.common import run_podman_command
 
@@ -424,6 +425,10 @@ class PodmanImageManager(object):
             self.absent()
 
     def _run(self, args, expected_rc=0, ignore_errors=False):
+        cmd = " ".join([self.executable]
+                       + [to_native(i) for i in args])
+        self.module.log("PODMAN-IMAGE-DEBUG: %s" % cmd)
+        self.results['podman_actions'].append(cmd)
         return run_podman_command(
             module=self.module,
             executable=self.executable,
@@ -457,16 +462,20 @@ class PodmanImageManager(object):
                 # Build the image
                 self.results['actions'].append('Built image {image_name} from {path}'.format(image_name=self.image_name, path=self.path))
                 if not self.module.check_mode:
-                    self.results['image'] = self.build_image()
+                    image = self.results['image'] = self.build_image()
             else:
                 # Pull the image
                 self.results['actions'].append('Pulled image {image_name}'.format(image_name=self.image_name))
                 if not self.module.check_mode:
-                    self.results['image'] = self.pull_image()
+                    image = self.results['image'] = self.pull_image()
 
-            image = self.find_image()
-            digest_after = image[0].get('Digest', image[0].get('digest'))
-            self.results['changed'] = digest_before != digest_after
+            if not image:
+                image = self.find_image()
+            if not self.module.check_mode:
+                digest_after = image[0].get('Digest', image[0].get('digest'))
+                self.results['changed'] = digest_before != digest_after
+            else:
+                self.results['changed'] = True
 
         if self.push:
             # Push the image
@@ -581,7 +590,7 @@ class PodmanImageManager(object):
 
         extra_args = self.build.get('extra_args')
         if extra_args:
-            args.extend([arg for arg in shlex.split(extra_args)])
+            args.extend(shlex.split(extra_args))
 
         args.append(self.path)
 
@@ -702,7 +711,7 @@ def main():
             path=dict(type='str'),
             force=dict(type='bool', default=False),
             state=dict(type='str', default='present', choices=['absent', 'present', 'build']),
-            validate_certs=dict(type='bool', default=True, aliases=['tlsverify', 'tls_verify']),
+            validate_certs=dict(type='bool', aliases=['tlsverify', 'tls_verify']),
             executable=dict(type='str', default='podman'),
             auth_file=dict(type='path', aliases=['authfile']),
             username=dict(type='str'),
@@ -714,7 +723,7 @@ def main():
                 default={},
                 options=dict(
                     annotation=dict(type='dict'),
-                    force_rm=dict(type='bool'),
+                    force_rm=dict(type='bool', default=False),
                     format=dict(
                         type='str',
                         choices=['oci', 'docker'],
@@ -761,6 +770,7 @@ def main():
     results = dict(
         changed=False,
         actions=[],
+        podman_actions=[],
         image={},
     )
 
