@@ -49,6 +49,11 @@ options:
     description:
       - Allocate container IP from range
     type: str
+  ipv6:
+    description:
+      - Enable IPv6 (Dual Stack) networking. You must pass a IPv6 subnet.
+        The subnet option must be used with the ipv6 option.
+    type: bool
   subnet:
     description:
       - Subnet in CIDR format
@@ -57,6 +62,21 @@ options:
     description:
       - Create a Macvlan connection based on this device
     type: str
+  opt:
+    description:
+      - Add network options. Currently 'vlan' and 'mtu' are supported.
+    type: dict
+    suboptions:
+      mtu:
+        description:
+          - MTU size for bridge network interface.
+        type: int
+        required: false
+      vlan:
+        description:
+          - VLAN tag for bridge which enables vlan_filtering.
+        type: int
+        required: false
   debug:
     description:
       - Return additional information which can be helpful for investigations.
@@ -87,7 +107,7 @@ EXAMPLES = r"""
   containers.podman.podman_network:
     name: podman_internal
     internal: true
-    ip_range: 192.168.22.50-192.168.22.150
+    ip_range: 192.168.22.128/25
     subnet: 192.168.22.0/24
     gateway: 192.168.22.1
   become: true
@@ -140,7 +160,6 @@ network:
         ]
 """
 
-# noqa: F402
 import json  # noqa: F402
 from distutils.version import LooseVersion  # noqa: F402
 import os  # noqa: F402
@@ -218,12 +237,22 @@ class PodmanNetworkModuleParams:
     def addparam_ip_range(self, c):
         return c + ['--ip-range', self.params['ip_range']]
 
+    def addparam_ipv6(self, c):
+        return c + ['--ipv6']
+
     def addparam_macvlan(self, c):
         return c + ['--macvlan', self.params['macvlan']]
 
     def addparam_internal(self, c):
         return c + ['--internal=%s' % self.params['internal']]
 
+    def addparam_opt(self, c):
+        for opt in self.params['opt'].items():
+            c += ['--opt',
+                  b"=".join([to_bytes(k, errors='surrogate_or_strict')
+                             for k in opt])]
+        return c
+
     def addparam_disable_dns(self, c):
         return c + ['--disable-dns=%s' % self.params['disable_dns']]
 
@@ -272,6 +301,12 @@ class PodmanNetworkDiff:
         return False
 
     def diffparam_disable_dns(self):
+        # Whether network is internal or not
+        try:
+            internal = not self.info['plugins'][0]['isgateway']
+        except (IndexError, KeyError):
+            internal = False
+        # Whether DNS plugin is installed
         dns_installed = False
         for f in [
             '/usr/libexec/cni/dnsname',
@@ -283,10 +318,15 @@ class PodmanNetworkDiff:
                 dns_installed = True
         before = not bool(
             [k for k in self.info['plugins'] if 'domainname' in k])
+        if internal:
+            before = True
         after = self.params['disable_dns']
         # If dnsname plugin is not installed, default is disable_dns=True
         if not dns_installed and self.module.params['disable_dns'] is None:
             after = True
+        # Internal networks have dns disabled from v3
+        if self.params['internal']:
+            after = True
         return self._diff_update_and_compare('disable_dns', before, after)
 
     def diffparam_driver(self):
@@ -331,6 +371,20 @@ class PodmanNetworkDiff:
         before = after = ''
         return self._diff_update_and_compare('macvlan', before, after)
 
+    def diffparam_opt(self):
+        vlan_before = self.info['plugins'][0].get('vlan')
+        vlan_after = self.params['opt'].get('vlan') if self.params['opt'] else None
+        if vlan_before or vlan_after:
+            before, after = {'vlan': vlan_before}, {'vlan': vlan_after}
+        else:
+            before, after = {}, {}
+        mtu_before = self.info['plugins'][0].get('mtu')
+        mtu_after = self.params['opt'].get('mtu') if self.params['opt'] else None
+        if mtu_before or mtu_after:
+            before.update({'mtu': mtu_before})
+            after.update({'mtu': mtu_after})
+        return self._diff_update_and_compare('opt', before, after)
+
     def is_different(self):
         diff_func_list = [func for func in dir(self)
                           if callable(getattr(self, func)) and func.startswith(
@@ -548,14 +602,20 @@ def main():
             gateway=dict(type='str', required=False),
             internal=dict(type='bool', required=False),
             ip_range=dict(type='str', required=False),
+            ipv6=dict(type='bool', required=False),
             subnet=dict(type='str', required=False),
             macvlan=dict(type='str', required=False),
+            opt=dict(type='dict', required=False,
+                     options=dict(
+                         mtu=dict(type='int', required=False),
+                         vlan=dict(type='int', required=False))),
             executable=dict(type='str', required=False, default='podman'),
             debug=dict(type='bool', default=False),
             recreate=dict(type='bool', default=False),
         ),
-        required_by=dict(  # for IP range to set 'subnet' is required
+        required_by=dict(  # for IP range and GW to set 'subnet' is required
             ip_range=('subnet'),
+            gateway=('subnet'),
         ))
 
     PodmanNetworkManager(module).execute()
