@@ -73,6 +73,10 @@ options:
     type: bool
     default: no
 extends_documentation_fragment: f5networks.f5_modules.f5
+notes:
+  - This module can be used to license BIG-IPs that do not have access to internet.
+  - Only the Ansible Controller needs internet access as the license activation is done on the Ansible Controller from
+    which the module is running.
 author:
   - Tim Rupp (@caphrim007)
   - Wojciech Wypior (@wojtek0806)
@@ -554,7 +558,8 @@ class ModuleManager(object):
                     data=self.want.license_envelope,
                 )
             except Exception:
-                continue
+                # Failures to connect to licensing server must be passed upstream not supressed
+                raise
 
             try:
                 resp = LicenseXmlParser(content=resp.content)
@@ -568,7 +573,8 @@ class ModuleManager(object):
                 # is an issue with the license server.
                 raise
             except Exception:
-                continue
+                # Any other exceptions must be raised also
+                raise
 
             if result['state'] == 'EULA_REQUIRED':
                 self.want.update({'eula': result['eula']})
