@@ -7,24 +7,19 @@
 from __future__ import absolute_import, division, print_function
 __metaclass__ = type
 
-
-ANSIBLE_METADATA = {'metadata_version': '1.1',
-                    'status': ['preview'],
-                    'supported_by': 'certified'}
-
 DOCUMENTATION = r'''
 ---
 module: bigip_gtm_wide_ip
-short_description: Manages F5 BIG-IP GTM wide ip
+short_description: Manages F5 BIG-IP GTM Wide IP
 description:
-  - Manages F5 BIG-IP GTM wide ip.
+  - Manages the F5 BIG-IP GTM (now BIG-IP DNS) Wide IP.
 version_added: "1.0.0"
 options:
   pool_lb_method:
     description:
       - Specifies the load balancing method used to select a pool in this wide
         IP. This setting is relevant only when multiple pools are configured
-        for a wide IP.
+        for a Wide IP.
     type: str
     aliases: ['lb_method']
     choices:
@@ -43,11 +38,11 @@ options:
       - wide_ip
   type:
     description:
-      - Specifies the type of wide IP. GTM wide IPs need to be keyed by query
-        type in addition to name, since pool members need different attributes
-        depending on the response RDATA they are meant to supply. This value
-        is required if you are using BIG-IP versions >= 12.0.0.
+      - Specifies the type of Wide IP. GTM Wide IPs need to be keyed by query
+        type in addition to name, because pool members need different attributes
+        depending on the response RDATA they are meant to supply.
     type: str
+    required: True
     choices:
       - a
       - aaaa
@@ -57,10 +52,10 @@ options:
       - srv
   state:
     description:
-      - When C(present) or C(enabled), ensures that the Wide IP exists and
+      - When C(present) or C(enabled), ensures the Wide IP exists and
         is enabled.
-      - When C(absent), ensures that the Wide IP has been removed.
-      - When C(disabled), ensures that the Wide IP exists and is disabled.
+      - When C(absent), ensures the Wide IP has been removed.
+      - When C(disabled), ensures the Wide IP exists and is disabled.
     type: str
     choices:
       - present
@@ -75,7 +70,7 @@ options:
     default: Common
   pools:
     description:
-      - The pools that you want associated with the Wide IP.
+      - The pools you want associated with the Wide IP.
       - If C(ratio) is not provided when creating a new Wide IP, it will default
         to 1.
     type: list
@@ -90,13 +85,13 @@ options:
         description:
           - Ratio for the pool.
           - The system uses this number with the Ratio load balancing method.
-          - When C(ratio) is not provided the module assigns it value of C(0)
+          - When C(ratio) is not provided, the module assigns it value of C(0).
         type: int
       order:
         description:
           - Order of the pool in relation to other pools attached to this Wide IP.
-          - Pool order is significant when the Global Availability load balancing method is utilized.
-          - When C(order) is not provided the module assigns it value of C(0)
+          - Pool order is significant when the Global Availability load balancing method is used.
+          - When C(order) is not provided, the module assigns it value of C(0).
         type: int
   irules:
     description:
@@ -110,13 +105,13 @@ options:
       - Specifies alternate domain names for the web site content you are load
         balancing.
       - You can use the same wildcard characters for aliases as you can for actual
-        wide IP names.
+        Wide IP names.
     type: list
     elements: str
   last_resort_pool:
     description:
-      - Specifies which GTM pool, for the system to use as the last resort pool for
-        the wide IP.
+      - Specifies which GTM pool for the system to use as the last resort pool for
+        the Wide IP.
       - The valid pools for this parameter are those with the C(type) specified in this
         module.
     type: str
@@ -145,8 +140,6 @@ options:
         allows one persistence record to be shared by LDNS addresses
         that match within this mask.
     type: int
-notes:
-  - Support for TMOS versions below v12.x has been deprecated for this module, and will be removed in f5_modules 1.4.
 extends_documentation_fragment: f5networks.f5_modules.f5
 author:
   - Tim Rupp (@caphrim007)
@@ -158,6 +151,7 @@ EXAMPLES = r'''
   bigip_gtm_wide_ip:
     pool_lb_method: round-robin
     name: my-wide-ip.example.com
+    type: a
     provider:
       user: admin
       password: secret
@@ -168,6 +162,7 @@ EXAMPLES = r'''
   bigip_gtm_wide_ip:
     pool_lb_method: round-robin
     name: my-wide-ip.example.com
+    type: a
     irules:
       - irule1
       - irule2
@@ -181,6 +176,7 @@ EXAMPLES = r'''
   bigip_gtm_wide_ip:
     pool_lb_method: round-robin
     name: my-wide-ip.example.com
+    type: a
     irules:
       - irule1
     provider:
@@ -193,6 +189,7 @@ EXAMPLES = r'''
   bigip_gtm_wide_ip:
     pool_lb_method: round-robin
     name: my-wide-ip.example.com
+    type: a
     irules: ""
     provider:
       user: admin
@@ -204,6 +201,7 @@ EXAMPLES = r'''
   bigip_gtm_wide_ip:
     pool_lb_method: round-robin
     name: my-wide-ip.example.com
+    type: a
     pools:
       - name: pool1
         ratio: 100
@@ -221,6 +219,7 @@ EXAMPLES = r'''
   bigip_gtm_wide_ip:
     pool_lb_method: round-robin
     name: my-wide-ip.example.com
+    type: a
     pools:
       - name: pool1
         persistence: yes
@@ -236,12 +235,12 @@ EXAMPLES = r'''
 
 RETURN = r'''
 lb_method:
-  description: The new load balancing method used by the wide IP.
+  description: The new load balancing method used by the Wide IP.
   returned: changed
   type: str
   sample: topology
 state:
-  description: The new state of the wide IP.
+  description: The new state of the Wide IP.
   returned: changed
   type: str
   sample: disabled
@@ -256,7 +255,7 @@ aliases:
   type: list
   sample: ['alias1.foo.com', '*.wildcard.domain']
 persistence:
-  description: Whether the pool connections will be persisted.
+  description: Whether pool connections will be persisted.
   returned: changed
   type: bool
   sample: False
@@ -276,12 +275,13 @@ persistence_ttl:
   type: int
   sample: 3600
 '''
+from datetime import datetime
+from distutils.version import LooseVersion
 
 from ansible.module_utils.basic import (
     AnsibleModule, env_fallback
 )
 from ansible.module_utils.six import iteritems
-from distutils.version import LooseVersion
 
 from ..module_utils.bigip import F5RestClient
 from ..module_utils.common import (
@@ -290,6 +290,7 @@ from ..module_utils.common import (
 from ..module_utils.icontrol import (
     module_provisioned, tmos_version
 )
+from ..module_utils.teem import send_teem
 
 
 class Parameters(AnsibleF5Parameters):
@@ -657,37 +658,6 @@ class Difference(object):
 
 
 class ModuleManager(object):
-    def __init__(self, *args, **kwargs):
-        self.module = kwargs.get('module', None)
-        self.client = F5RestClient(**self.module.params)
-        self.kwargs = kwargs
-
-    def exec_module(self):
-        if not module_provisioned(self.client, 'gtm'):
-            raise F5ModuleError(
-                "GTM must be provisioned to use this module."
-            )
-        if self.version_is_less_than_12():
-            manager = self.get_manager('untyped')
-        else:
-            manager = self.get_manager('typed')
-        return manager.exec_module()
-
-    def get_manager(self, type):
-        if type == 'typed':
-            return TypedManager(**self.kwargs)
-        elif type == 'untyped':
-            return UntypedManager(**self.kwargs)
-
-    def version_is_less_than_12(self):
-        version = tmos_version(self.client)
-        if LooseVersion(version) < LooseVersion('12.0.0'):
-            return True
-        else:
-            return False
-
-
-class BaseManager(object):
     def __init__(self, *args, **kwargs):
         self.module = kwargs.get('module', None)
         self.client = F5RestClient(**self.module.params)
@@ -722,6 +692,12 @@ class BaseManager(object):
         return False
 
     def exec_module(self):
+        if not module_provisioned(self.client, 'gtm'):
+            raise F5ModuleError(
+                "GTM must be provisioned to use this module."
+            )
+        start = datetime.now().isoformat()
+        version = tmos_version(self.client)
         changed = False
         result = dict()
         state = self.want.state
@@ -736,33 +712,17 @@ class BaseManager(object):
         result.update(**changes)
         result.update(dict(changed=changed))
         self._announce_deprecations(result)
+        send_teem(start, self.module, version)
         return result
 
     def _announce_deprecations(self, result):
         warnings = result.pop('__warnings', [])
-        if self.version_is_less_than_12():
-            self._deprecate_v11(warnings)
         for warning in warnings:
             self.module.deprecate(
                 msg=warning['msg'],
                 version=warning['version']
             )
 
-    def version_is_less_than_12(self):
-        version = tmos_version(self.client)
-        if LooseVersion(version) < LooseVersion('12.0.0'):
-            return True
-        else:
-            return False
-
-    def _deprecate_v11(self, result):
-        result.append(
-            dict(
-                msg='The support for this TMOS version is deprecated.',
-                version='f5_modules 1.4'
-            )
-        )
-
     def present(self):
         if self.exists():
             return self.update()
@@ -808,113 +768,6 @@ class BaseManager(object):
             raise F5ModuleError("Failed to delete the Wide IP")
         return True
 
-
-class UntypedManager(BaseManager):
-    def exists(self):
-        uri = "https://{0}:{1}/mgmt/tm/gtm/wideip/{2}".format(
-            self.client.provider['server'],
-            self.client.provider['server_port'],
-            transform_name(self.want.partition, self.want.name)
-        )
-        resp = self.client.api.get(uri)
-        try:
-            response = resp.json()
-        except ValueError as ex:
-            raise F5ModuleError(str(ex))
-
-        if resp.status == 404 or 'code' in response and response['code'] == 404:
-            return False
-        if resp.status in [200, 201] or 'code' in response and response['code'] in [200, 201]:
-            return True
-
-        errors = [401, 403, 409, 500, 501, 502, 503, 504]
-
-        if resp.status in errors or 'code' in response and response['code'] in errors:
-            if 'message' in response:
-                raise F5ModuleError(response['message'])
-            else:
-                raise F5ModuleError(resp.content)
-
-    def update_on_device(self):
-        params = self.changes.api_params()
-        uri = "https://{0}:{1}/mgmt/tm/gtm/wideip/{2}".format(
-            self.client.provider['server'],
-            self.client.provider['server_port'],
-            transform_name(self.want.partition, self.want.name)
-        )
-        resp = self.client.api.patch(uri, json=params)
-        try:
-            response = resp.json()
-        except ValueError as ex:
-            raise F5ModuleError(str(ex))
-
-        if 'code' in response and response['code'] == 400:
-            if 'message' in response:
-                raise F5ModuleError(response['message'])
-            else:
-                raise F5ModuleError(resp.content)
-
-    def read_current_from_device(self):
-        uri = "https://{0}:{1}/mgmt/tm/gtm/wideip/{2}".format(
-            self.client.provider['server'],
-            self.client.provider['server_port'],
-            transform_name(self.want.partition, self.want.name)
-        )
-        resp = self.client.api.get(uri)
-        try:
-            response = resp.json()
-        except ValueError as ex:
-            raise F5ModuleError(str(ex))
-
-        if 'code' in response and response['code'] == 400:
-            if 'message' in response:
-                raise F5ModuleError(response['message'])
-            else:
-                raise F5ModuleError(resp.content)
-        return ApiParameters(params=response)
-
-    def create_on_device(self):
-        params = self.changes.api_params()
-        params['name'] = self.want.name
-        params['partition'] = self.want.partition
-        uri = "https://{0}:{1}/mgmt/tm/gtm/wideip/".format(
-            self.client.provider['server'],
-            self.client.provider['server_port']
-        )
-        resp = self.client.api.post(uri, json=params)
-        try:
-            response = resp.json()
-        except ValueError as ex:
-            raise F5ModuleError(str(ex))
-
-        if 'code' in response and response['code'] in [400, 403]:
-            if 'message' in response:
-                raise F5ModuleError(response['message'])
-            else:
-                raise F5ModuleError(resp.content)
-        return response['selfLink']
-
-    def remove_from_device(self):
-        uri = "https://{0}:{1}/mgmt/tm/gtm/wideip/{2}".format(
-            self.client.provider['server'],
-            self.client.provider['server_port'],
-            transform_name(self.want.partition, self.want.name)
-        )
-        response = self.client.api.delete(uri)
-        if response.status == 200:
-            return True
-        raise F5ModuleError(response.content)
-
-
-class TypedManager(BaseManager):
-    def __init__(self, *args, **kwargs):
-        super(TypedManager, self).__init__(**kwargs)
-        if self.want.type is None:
-            raise F5ModuleError(
-                "The 'type' option is required for BIG-IP instances "
-                "greater than or equal to 12.x"
-            )
-
     def exists(self):
         uri = "https://{0}:{1}/mgmt/tm/gtm/wideip/{2}/{3}".format(
             self.client.provider['server'],
@@ -1026,7 +879,8 @@ class ArgumentSpec(object):
             type=dict(
                 choices=[
                     'a', 'aaaa', 'cname', 'mx', 'naptr', 'srv'
-                ]
+                ],
+                required=True
             ),
             state=dict(
                 default='present',
