@@ -49,6 +49,11 @@ options:
             - List of Volumes IDs that should be attached to the server on server creation.
         type: list
         elements: str
+    firewalls:
+        description:
+            - List of Firewall IDs that should be attached to the server on server creation.
+        type: list
+        elements: str
     image:
         description:
             - Image the server should be created from.
@@ -255,12 +260,15 @@ try:
     from hcloud.volumes.domain import Volume
     from hcloud.ssh_keys.domain import SSHKey
     from hcloud.servers.domain import Server
+    from hcloud.firewalls.domain import Firewall, FirewallResource
     from hcloud import APIException
 except ImportError:
     APIException = None
     Volume = None
     SSHKey = None
     Server = None
+    Firewall = None
+    FirewallResource = None
 
 
 class AnsibleHcloudServer(Hcloud):
@@ -297,7 +305,7 @@ class AnsibleHcloudServer(Hcloud):
                 self.hcloud_server = self.client.servers.get_by_name(
                     self.module.params.get("name")
                 )
-        except APIException as e:
+        except Exception as e:
             self.module.fail_json(msg=e.message)
 
     def _create_server(self):
@@ -314,9 +322,10 @@ class AnsibleHcloudServer(Hcloud):
             "user_data": self.module.params.get("user_data"),
             "labels": self.module.params.get("labels"),
         }
-        if self.client.images.get_by_name(self.module.params.get("image")) is not None:
+        image = self.client.images.get_by_name(self.module.params.get("image"))
+        if image is not None:
             # When image name is not available look for id instead
-            params["image"] = self.client.images.get_by_name(self.module.params.get("image"))
+            params["image"] = image
         else:
             params["image"] = self.client.images.get_by_id(self.module.params.get("image"))
 
@@ -330,6 +339,15 @@ class AnsibleHcloudServer(Hcloud):
             params["volumes"] = [
                 Volume(id=volume_id) for volume_id in self.module.params.get("volumes")
             ]
+        if self.module.params.get("firewalls") is not None:
+            params["firewalls"] = []
+            for fw in self.module.params.get("firewalls"):
+                f = self.client.firewalls.get_by_name(fw)
+                if f is not None:
+                    # When firewall name is not available look for id instead
+                    params["firewalls"].append(f)
+                else:
+                    params["firewalls"].append(self.client.firewalls.get_by_id(fw))
 
         if self.module.params.get("location") is None and self.module.params.get("datacenter") is None:
             # When not given, the API will choose the location.
@@ -345,27 +363,30 @@ class AnsibleHcloudServer(Hcloud):
             )
 
         if not self.module.check_mode:
-            resp = self.client.servers.create(**params)
-            self.result["root_password"] = resp.root_password
-            resp.action.wait_until_finished(max_retries=1000)
-            [action.wait_until_finished() for action in resp.next_actions]
+            try:
+                resp = self.client.servers.create(**params)
+                self.result["root_password"] = resp.root_password
+                resp.action.wait_until_finished(max_retries=1000)
+                [action.wait_until_finished() for action in resp.next_actions]
 
-            rescue_mode = self.module.params.get("rescue_mode")
-            if rescue_mode:
-                self._get_server()
-                self._set_rescue_mode(rescue_mode)
+                rescue_mode = self.module.params.get("rescue_mode")
+                if rescue_mode:
+                    self._get_server()
+                    self._set_rescue_mode(rescue_mode)
 
-            backups = self.module.params.get("backups")
-            if backups:
-                self._get_server()
-                self.hcloud_server.enable_backup().wait_until_finished()
+                backups = self.module.params.get("backups")
+                if backups:
+                    self._get_server()
+                    self.hcloud_server.enable_backup().wait_until_finished()
 
-            delete_protection = self.module.params.get("delete_protection")
-            rebuild_protection = self.module.params.get("rebuild_protection")
-            if delete_protection is not None and rebuild_protection is not None:
-                self._get_server()
-                self.hcloud_server.change_protection(delete=delete_protection,
-                                                     rebuild=rebuild_protection).wait_until_finished()
+                delete_protection = self.module.params.get("delete_protection")
+                rebuild_protection = self.module.params.get("rebuild_protection")
+                if delete_protection is not None and rebuild_protection is not None:
+                    self._get_server()
+                    self.hcloud_server.change_protection(delete=delete_protection,
+                                                         rebuild=rebuild_protection).wait_until_finished()
+            except Exception as e:
+                self.module.fail_json(msg=e.message)
         self._mark_as_changed()
         self._get_server()
 
@@ -396,6 +417,34 @@ class AnsibleHcloudServer(Hcloud):
                     self.hcloud_server.update(labels=labels)
                 self._mark_as_changed()
 
+            firewalls = self.module.params.get("firewalls")
+            if firewalls is not None and firewalls != self.hcloud_server.public_net.firewalls:
+                if not self.module.check_mode:
+                    for f in self.hcloud_server.public_net.firewalls:
+                        found = False
+                        for fname in firewalls:
+                            if f.firewall.name == fname:
+                                found = True
+                        if not found:
+                            r = FirewallResource(type="server", server=self.hcloud_server)
+                            actions = self.client.firewalls.remove_from_resources(f.firewall, [r])
+                            for a in actions:
+                                a.wait_until_finished()
+
+                    for fname in firewalls:
+                        found = False
+                        fw = None
+                        for f in self.hcloud_server.public_net.firewalls:
+                            if f.firewall.name == fname:
+                                found = True
+                                fw = f
+                        if not found and fw is not None:
+                            r = FirewallResource(type="server", server=self.hcloud_server)
+                            actions = self.client.firewalls.apply_to_resources(fw, [r])
+                            for a in actions:
+                                a.wait_until_finished()
+                self._mark_as_changed()
+
             server_type = self.module.params.get("server_type")
             if server_type is not None and self.hcloud_server.server_type.name != server_type:
                 previous_server_status = self.hcloud_server.status
@@ -434,7 +483,7 @@ class AnsibleHcloudServer(Hcloud):
                                                          rebuild=rebuild_protection).wait_until_finished()
                 self._mark_as_changed()
             self._get_server()
-        except APIException as e:
+        except Exception as e:
             self.module.fail_json(msg=e.message)
 
     def _set_rescue_mode(self, rescue_mode):
@@ -451,22 +500,24 @@ class AnsibleHcloudServer(Hcloud):
 
     def start_server(self):
         try:
-            if self.hcloud_server.status != Server.STATUS_RUNNING:
-                if not self.module.check_mode:
-                    self.client.servers.power_on(self.hcloud_server).wait_until_finished()
-                self._mark_as_changed()
-            self._get_server()
-        except APIException as e:
+            if self.hcloud_server:
+                if self.hcloud_server.status != Server.STATUS_RUNNING:
+                    if not self.module.check_mode:
+                        self.client.servers.power_on(self.hcloud_server).wait_until_finished()
+                    self._mark_as_changed()
+                self._get_server()
+        except Exception as e:
             self.module.fail_json(msg=e.message)
 
     def stop_server(self):
         try:
-            if self.hcloud_server.status != Server.STATUS_OFF:
-                if not self.module.check_mode:
-                    self.client.servers.power_off(self.hcloud_server).wait_until_finished()
-                self._mark_as_changed()
-            self._get_server()
-        except APIException as e:
+            if self.hcloud_server:
+                if self.hcloud_server.status != Server.STATUS_OFF:
+                    if not self.module.check_mode:
+                        self.client.servers.power_off(self.hcloud_server).wait_until_finished()
+                    self._mark_as_changed()
+                self._get_server()
+        except Exception as e:
             self.module.fail_json(msg=e.message)
 
     def rebuild_server(self):
@@ -480,7 +531,7 @@ class AnsibleHcloudServer(Hcloud):
             self._mark_as_changed()
 
             self._get_server()
-        except APIException as e:
+        except Exception as e:
             self.module.fail_json(msg=e.message)
 
     def present_server(self):
@@ -498,7 +549,7 @@ class AnsibleHcloudServer(Hcloud):
                     self.client.servers.delete(self.hcloud_server).wait_until_finished()
                 self._mark_as_changed()
             self.hcloud_server = None
-        except APIException as e:
+        except Exception as e:
             self.module.fail_json(msg=e.message)
 
     @staticmethod
@@ -514,6 +565,7 @@ class AnsibleHcloudServer(Hcloud):
                 user_data={"type": "str"},
                 ssh_keys={"type": "list", "elements": "str"},
                 volumes={"type": "list", "elements": "str"},
+                firewalls={"type": "list", "elements": "str"},
                 labels={"type": "dict"},
                 backups={"type": "bool", "default": False},
                 upgrade_disk={"type": "bool", "default": False},
