@@ -34,6 +34,8 @@ netapp.py
 from __future__ import (absolute_import, division, print_function)
 __metaclass__ = type
 
+import base64
+import logging
 import os
 import ssl
 import time
@@ -45,13 +47,16 @@ try:
 except ImportError:
     ansible_version = 'unknown'
 
-COLLECTION_VERSION = "20.11.0"
+COLLECTION_VERSION = "21.3.1"
+CLIENT_APP_VERSION = "%s/" + COLLECTION_VERSION
+IMPORT_EXCEPTION = None
 
 try:
     from netapp_lib.api.zapi import zapi
     HAS_NETAPP_LIB = True
-except ImportError:
+except ImportError as exc:
     HAS_NETAPP_LIB = False
+    IMPORT_EXCEPTION = exc
 
 try:
     import requests
@@ -92,6 +97,8 @@ ERROR_MSG = dict(
     no_cserver='This module is expected to run as cluster admin'
 )
 
+LOG = logging.getLogger(__name__)
+
 try:
     from solidfire.factory import ElementFactory
     HAS_SF_SDK = True
@@ -103,6 +110,10 @@ def has_netapp_lib():
     return HAS_NETAPP_LIB
 
 
+def netapp_lib_is_required():
+    return "Error: the python NetApp-Lib module is required.  Import error: %s" % str(IMPORT_EXCEPTION)
+
+
 def has_sf_sdk():
     return HAS_SF_SDK
 
@@ -137,13 +148,17 @@ def get_feature(module, feature_name):
     '''
     default_flags = dict(
         check_required_params_for_none=True,
+        classic_basic_authorization=False,      # use ZAPI wrapper to send Authorization header
         deprecation_warning=True,
         sanitize_xml=True,
-        sanitize_code_points=[8],       # unicode values, 8 is backspace
-        show_modified=True
+        sanitize_code_points=[8],               # unicode values, 8 is backspace
+        show_modified=True,
+        always_wrap_zapi=True,                  # for better error reporting
+        trace_apis=False,                       # if true, append ZAPI and REST requests/responses to /tmp/ontap_zapi.txt
+        flexcache_delete_return_timeout=5       # ONTAP bug if too big?
     )
 
-    if feature_name in module.params['feature_flags']:
+    if module.params['feature_flags'] is not None and feature_name in module.params['feature_flags']:
         return module.params['feature_flags'][feature_name]
     if feature_name in default_flags:
         return default_flags[feature_name]
@@ -161,8 +176,7 @@ def create_sf_connection(module, port=None):
             return return_val
         except Exception:
             raise Exception("Unable to create SF connection")
-    else:
-        module.fail_json(msg="the python SolidFire SDK module is required")
+    module.fail_json(msg="the python SolidFire SDK module is required")
 
 
 def set_auth_method(module, username, password, cert_filepath, key_filepath):
@@ -180,8 +194,10 @@ def set_auth_method(module, username, password, cert_filepath, key_filepath):
         if cert_filepath is not None or key_filepath is not None:
             error = 'Error: cannot have both basic authentication (username/password) ' +\
                     'and certificate authentication (cert/key files)'
-        else:
+        elif has_feature(module, 'classic_basic_authorization'):
             auth_method = 'basic_auth'
+        else:
+            auth_method = 'speedy_basic_auth'
     else:
         error = 'Error: username and password have to be provided together'
         if cert_filepath is not None or key_filepath is not None:
@@ -202,24 +218,32 @@ def setup_na_ontap_zapi(module, vserver=None, wrap_zapi=False):
     cert_filepath = module.params['cert_filepath']
     key_filepath = module.params['key_filepath']
     auth_method = set_auth_method(module, username, password, cert_filepath, key_filepath)
+    if has_feature(module, 'trace_apis'):
+        logging.basicConfig(filename='/tmp/ontap_apis.log', level=logging.DEBUG)
+        trace = True
+    else:
+        trace = False
+    if has_feature(module, 'always_wrap_zapi'):
+        wrap_zapi = True
 
     if HAS_NETAPP_LIB:
         # set up zapi
-        if auth_method != 'basic_auth':
+        if auth_method in ('single_cert', 'cert_key'):
             # override NaServer in netapp-lib to enable certificate authentication
             server = OntapZAPICx(hostname, module=module, username=username, password=password,
                                  validate_certs=validate_certs, cert_filepath=cert_filepath,
-                                 key_filepath=key_filepath, style=zapi.NaServer.STYLE_CERTIFICATE)
+                                 key_filepath=key_filepath, style=zapi.NaServer.STYLE_CERTIFICATE,
+                                 auth_method=auth_method, trace=trace)
             # SSL certificate authentication requires SSL
             https = True
-        elif wrap_zapi:
+        elif auth_method == 'speedy_basic_auth' or wrap_zapi:
+            # override NaServer in netapp-lib to add Authorization header preemptively
+            # use wrapper to handle parse error (mostly for na_ontap_command)
             server = OntapZAPICx(hostname, module=module, username=username, password=password,
-                                 validate_certs=validate_certs)
+                                 validate_certs=validate_certs, auth_method=auth_method, trace=trace)
         else:
             # legacy netapp-lib
-            server = zapi.NaServer(hostname)
-            server.set_username(username)
-            server.set_password(password)
+            server = zapi.NaServer(hostname, username=username, password=password, trace=trace)
         if vserver:
             server.set_vserver(vserver)
         if version:
@@ -245,7 +269,7 @@ def setup_na_ontap_zapi(module, vserver=None, wrap_zapi=False):
         server.set_server_type('FILER')
         return server
     else:
-        module.fail_json(msg="the python NetApp-Lib module is required")
+        module.fail_json(msg=netapp_lib_is_required())
 
 
 def is_zapi_connection_error(message):
@@ -364,12 +388,14 @@ if HAS_NETAPP_LIB:
         ''' override zapi NaServer class to:
         - enable SSL certificate authentication
         - ignore invalid XML characters in ONTAP output (when using CLI module)
+        - add Authorization header when using basic authentication
         '''
         def __init__(self, hostname=None, server_type=zapi.NaServer.SERVER_TYPE_FILER,
                      transport_type=zapi.NaServer.TRANSPORT_TYPE_HTTP,
                      style=zapi.NaServer.STYLE_LOGIN_PASSWORD, username=None,
                      password=None, port=None, trace=False, module=None,
-                     cert_filepath=None, key_filepath=None, validate_certs=None):
+                     cert_filepath=None, key_filepath=None, validate_certs=None,
+                     auth_method=None):
             # python 2.x syntax, but works for python 3 as well
             super(OntapZAPICx, self).__init__(hostname, server_type=server_type,
                                               transport_type=transport_type,
@@ -379,6 +405,10 @@ if HAS_NETAPP_LIB:
             self.key_filepath = key_filepath
             self.validate_certs = validate_certs
             self.module = module
+            self.base64_creds = None
+            if auth_method == 'speedy_basic_auth':
+                auth = '%s:%s' % (username, password)
+                self.base64_creds = base64.b64encode(auth.encode()).decode()
 
         def _create_certificate_auth_handler(self):
             try:
@@ -430,6 +460,58 @@ if HAS_NETAPP_LIB:
                     pass
                 raise exc
 
+        def _create_request(self, na_element, enable_tunneling=False):
+            ''' intercept newly created request to add Authorization header '''
+            request, netapp_element = super(OntapZAPICx, self)._create_request(na_element, enable_tunneling=enable_tunneling)
+            request.add_header('X-Dot-Client-App', CLIENT_APP_VERSION % self.module._name)
+            if self.base64_creds is not None:
+                request.add_header('Authorization', 'Basic %s' % self.base64_creds)
+            return request, netapp_element
+
+        # as is from latest version of netapp-lib
+        def invoke_elem(self, na_element, enable_tunneling=False):
+            """Invoke the API on the server."""
+            if not na_element or not isinstance(na_element, zapi.NaElement):
+                raise ValueError('NaElement must be supplied to invoke API')
+
+            request, request_element = self._create_request(na_element,
+                                                            enable_tunneling)
+
+            if self._trace:
+                zapi.LOG.debug("Request: %s", request_element.to_string(pretty=True))
+
+            if not hasattr(self, '_opener') or not self._opener \
+                    or self._refresh_conn:
+                self._build_opener()
+            try:
+                if hasattr(self, '_timeout'):
+                    response = self._opener.open(request, timeout=self._timeout)
+                else:
+                    response = self._opener.open(request)
+            except zapi.urllib.error.HTTPError as exc:
+                raise zapi.NaApiError(exc.code, exc.reason)
+            except zapi.urllib.error.URLError as exc:
+                msg = 'URL error'
+                error = repr(exc)
+                try:
+                    # ConnectionRefusedError is not defined in python 2.7
+                    if isinstance(exc.reason, ConnectionRefusedError):
+                        msg = 'Unable to connect'
+                        error = exc.args
+                except Exception:
+                    pass
+                raise zapi.NaApiError(msg, error)
+            except Exception as exc:
+                raise zapi.NaApiError('Unexpected error', repr(exc))
+
+            response_xml = response.read()
+            response_element = self._get_result(response_xml)
+
+            if self._trace:
+                zapi.LOG.debug("Response: %s", response_element.to_string(pretty=True))
+
+            return response_element
+
 
 class OntapRestAPI(object):
     ''' wrapper to send requests to ONTAP REST APIs '''
@@ -460,18 +542,52 @@ class OntapRestAPI(object):
         self.debug_logs = list()
         self.auth_method = set_auth_method(self.module, self.username, self.password, self.cert_filepath, self.key_filepath)
         self.check_required_library()
+        if has_feature(module, 'trace_apis'):
+            logging.basicConfig(filename='/tmp/ontap_apis.log', level=logging.DEBUG, format='%(asctime)s %(levelname)-8s %(message)s')
 
     def requires_ontap_9_6(self, module_name):
         self.requires_ontap_version(module_name)
 
     def requires_ontap_version(self, module_name, version='9.6'):
         suffix = " - %s" % self.is_rest_error if self.is_rest_error is not None else ""
-        return "%s only support REST, and requires ONTAP %s or later.%s" % (module_name, version, suffix)
+        return "%s only supports REST, and requires ONTAP %s or later.%s" % (module_name, version, suffix)
+
+    def options_require_ontap_version(self, options, version='9.6'):
+        current_version = self.get_ontap_version()
+        suffix = " - %s" % self.is_rest_error if self.is_rest_error is not None else ""
+        if current_version != (-1, -1):
+            suffix += " - ONTAP version: %s.%s" % current_version
+        if isinstance(options, list):
+            if len(options) > 1:
+                tag = "any of %s" % options
+            elif len(options) == 1:
+                tag = str(options[0])
+            else:
+                tag = str(options)
+        else:
+            tag = str(options)
+        return 'using %s requires ONTAP %s or later and REST must be enabled.%s' % (tag, version, suffix)
+
+    def meets_rest_minimum_version(self, use_rest, minimum_generation, minimum_major):
+        return use_rest and self.get_ontap_version() >= (minimum_generation, minimum_major)
 
     def check_required_library(self):
         if not HAS_REQUESTS:
             self.module.fail_json(msg=missing_required_lib('requests'))
 
+    def build_headers(self, accept=None, vserver_name=None, vserver_uuid=None):
+        headers = dict()
+        headers['X-Dot-Client-App'] = CLIENT_APP_VERSION % self.module._name
+        # accept is used to turn on/off HAL linking
+        if accept is not None:
+            headers['accept'] = accept
+        # vserver tunneling using vserver name and/or UUID
+        if vserver_name is not None:
+            headers['X-Dot-SVM-Name'] = vserver_name
+        if vserver_uuid is not None:
+            headers['X-Dot-SVM-UUID'] = vserver_uuid
+        return headers
+
     def send_request(self, method, api, params, json=None, accept=None,
                      vserver_name=None, vserver_uuid=None):
         ''' send http request and process reponse, including error conditions '''
@@ -481,17 +597,7 @@ class OntapRestAPI(object):
         json_dict = None
         json_error = None
         error_details = None
-        headers = None
-        if accept is not None or vserver_name is not None or vserver_uuid is not None:
-            headers = dict()
-            # accept is used to turn on/off HAL linking
-            if accept is not None:
-                headers['accept'] = accept
-            # vserver tunneling using vserver name and/or UUID
-            if vserver_name is not None:
-                headers['X-Dot-SVM-Name'] = vserver_name
-            if vserver_uuid is not None:
-                headers['X-Dot-SVM-UUID'] = vserver_uuid
+        headers = self.build_headers(accept, vserver_name, vserver_uuid)
 
         def get_json(response):
             ''' extract json, and error message if present '''
@@ -506,11 +612,14 @@ class OntapRestAPI(object):
             kwargs = dict(cert=self.cert_filepath)
         elif self.auth_method == 'cert_key':
             kwargs = dict(cert=(self.cert_filepath, self.key_filepath))
-        elif self.auth_method == 'basic_auth':
+        elif self.auth_method in ('basic_auth', 'speedy_basic_auth'):
+            # with requests, there is no challenge, eg no 401.
             kwargs = dict(auth=(self.username, self.password))
         else:
             raise KeyError(self.auth_method)
 
+        self.log_debug('sending', repr(dict(method=method, url=url, verify=self.verify, params=params,
+                                            timeout=self.timeout, json=json, headers=headers, **kwargs)))
         try:
             response = requests.request(method, url, verify=self.verify, params=params,
                                         timeout=self.timeout, json=json, headers=headers, **kwargs)
@@ -589,9 +698,9 @@ class OntapRestAPI(object):
 
                 message = job_json.get('message', '')
                 if job_json['state'] == 'failure':
-                    # if the job as failed, return message as error
+                    # if the job has failed, return message as error
                     return None, message
-                if job_json['state'] != 'running':
+                if job_json['state'] not in ('queued', 'running'):
                     keep_running = False
                 else:
                     # Would like to post a message to user (not sure how)
@@ -604,7 +713,7 @@ class OntapRestAPI(object):
                 runtime += increment
         return message, error
 
-    def get(self, api, params):
+    def get(self, api, params=None):
         method = 'GET'
         dummy, message, error = self.send_request(method, api, params)
         return message, error
@@ -633,43 +742,64 @@ class OntapRestAPI(object):
         try:
             version = message.get('version', 'not found')
         except AttributeError:
+            self.ontap_version['valid'] = False
             self.ontap_version['full'] = 'unreadable message'
             return
         for key in self.ontap_version:
             try:
                 self.ontap_version[key] = version.get(key, -1)
             except AttributeError:
-                self.ontap_version[key] = 'unreadable version'
+                self.ontap_version[key] = -1
         self.ontap_version['valid'] = True
         for key in self.ontap_version:
-            if self.ontap_version == -1:
+            if self.ontap_version[key] == -1:
                 self.ontap_version['valid'] = False
                 break
 
-    def _is_rest(self, used_unsupported_rest_properties=None):
-        if self.use_rest not in ['always', 'auto', 'never']:
-            error = "use_rest must be one of: never, always, auto. Got: '%s'" % self.use_rest
-            return False, error
-        if self.use_rest == "always":
-            if used_unsupported_rest_properties:
-                error = "REST API currently does not support '%s'" % \
-                        ', '.join(used_unsupported_rest_properties)
-                return True, error
-            else:
-                return True, None
-        if self.use_rest == 'never' or used_unsupported_rest_properties:
-            # force ZAPI if requested or if some parameter requires it
-            return False, None
-        # using GET rather than HEAD because the error messages are different
+    def get_ontap_version(self):
+        if self.ontap_version['valid']:
+            return self.ontap_version['generation'], self.ontap_version['major']
+        return -1, -1
+
+    def get_ontap_version_using_rest(self):
+        # using GET rather than HEAD because the error messages are different,
+        # and we need the version as some REST options are not available in earlier versions
         method = 'GET'
         api = 'cluster'
         params = {'fields': ['version']}
         status_code, message, error = self.send_request(method, api, params=params)
         self.set_version(message)
         self.is_rest_error = str(error) if error else None
+        if error:
+            self.log_error(status_code, str(error))
+        return status_code
+
+    def _is_rest(self, used_unsupported_rest_properties=None):
+        if self.use_rest not in ['always', 'auto', 'never']:
+            error = "use_rest must be one of: never, always, auto. Got: '%s'" % self.use_rest
+            return False, error
+        if self.use_rest == "always" and used_unsupported_rest_properties:
+            error = "REST API currently does not support '%s'" % \
+                    ', '.join(used_unsupported_rest_properties)
+            return True, error
+        if self.use_rest == 'never':
+            # force ZAPI if requested
+            return False, None
+        status_code = self.get_ontap_version_using_rest()
+        if self.use_rest == 'always':
+            # ignore error, it will show up later when calling another REST API
+            return True, None
+        # we're now using 'auto'
+        if used_unsupported_rest_properties:
+            # force ZAPI if some parameter requires it
+            if self.get_ontap_version() > (9, 5):
+                self.module.warn('Falling back to ZAPI because of unsupported option(s) or option value(s) in REST: %s' % used_unsupported_rest_properties)
+            return False, None
+        if self.get_ontap_version() in ((9, 4), (9, 5)):
+            # we can't trust REST support on 9.5, and not at all on 9.4
+            return False, None
         if status_code == 200:
             return True, None
-        self.log_error(status_code, str(error))
         return False, None
 
     def is_rest(self, used_unsupported_rest_properties=None):
@@ -680,10 +810,12 @@ class OntapRestAPI(object):
         return use_rest, error
 
     def log_error(self, status_code, message):
+        LOG.error("%s: %s", status_code, message)
         self.errors.append(message)
         self.debug_logs.append((status_code, message))
 
     def log_debug(self, status_code, content):
+        LOG.debug("%s: %s", status_code, content)
         self.debug_logs.append((status_code, content))
 
     def write_to_file(self, tag, data=None, filepath=None, append=True):
