@@ -40,6 +40,7 @@ options:
   name:
     description:
     - The name of the LUN to manage.
+    - Or LUN group name (volume name) when san_application_template is used.
     required: true
     type: str
 
@@ -69,6 +70,12 @@ options:
     default: 'gb'
     type: str
 
+  comment:
+    description:
+    - Optional descriptive comment for the LUN.
+    type: str
+    version_added: 21.2.0
+
   force_resize:
     description:
       Forcibly reduce the size. This is required for reducing the size of the LUN to avoid accidentally
@@ -109,6 +116,14 @@ options:
     type: str
     version_added: 20.12.0
 
+  qos_adaptive_policy_group:
+    description:
+    - The adaptive QoS policy group to be set on the LUN.
+    - Defines measurable service level objectives (SLOs) and service level agreements (SLAs) that adjust based on the LUN's allocated space or used space.
+    - Requires ONTAP 9.4 or later.
+    type: str
+    version_added: 21.2.0
+
   space_reserve:
     description:
     - This can be set to "false" which will create a LUN without any space being reserved.
@@ -133,9 +148,8 @@ options:
   san_application_template:
     description:
       - additional options when using the application/applications REST API to create LUNs.
-      - the module is using ZAPI by default, and switches to REST if any suboption is present.
+      - the module is using ZAPI by default, and switches to REST if san_application_template is present.
       - create one or more LUNs (and the associated volume as needed).
-      - only creation or deletion of a SAN application is supported.  Changes are ignored.
       - operations at the LUN level are supported, they require to know the LUN short name.
       - this requires ONTAP 9.6 or higher.
     type: dict
@@ -204,7 +218,17 @@ options:
           - This will default to true if any other suboption is present.
         type: bool
         default: true
-
+      scope:
+        description:
+          - whether the top level name identifies a single LUN or a LUN group (application).
+          - By default, the module will try to make the right choice, but can report extra warnings.
+          - Setting scope to 'application' is required to convert an existing volume to a smart container.
+          - The module reports an error when 'lun' or 'application' is used and the desired action cannot be completed.
+          - The module issues warnings when the default 'auto' is used, and there is ambiguity regarding the desired actions.
+        type: str
+        choices: ['application', 'auto', 'lun']
+        default: auto
+        version_added: 21.2.0
 '''
 
 EXAMPLES = """
@@ -253,6 +277,26 @@ EXAMPLES = """
     hostname: "{{ netapp_hostname }}"
     username: "{{ netapp_username }}"
     password: "{{ netapp_password }}"
+
+- name: Convert existing volume to SAN application
+  tags: create
+  na_ontap_lun:
+    state: present
+    name: someVolume
+    size: 22
+    size_unit: mb
+    os_type: linux
+    space_reserve: false
+    san_application_template:
+      name: san-ansibleLUN
+      igroup_name: testme_igroup
+      lun_count: 3
+      protection_type:
+      local_policy: default
+      scope: application
+    hostname: "{{ netapp_hostname }}"
+    username: "{{ netapp_username }}"
+    password: "{{ netapp_password }}"
 """
 
 RETURN = """
@@ -267,6 +311,7 @@ from ansible.module_utils._text import to_native
 import ansible_collections.netapp.ontap.plugins.module_utils.netapp as netapp_utils
 from ansible_collections.netapp.ontap.plugins.module_utils.netapp_module import NetAppModule
 from ansible_collections.netapp.ontap.plugins.module_utils.rest_application import RestApplication
+import ansible_collections.netapp.ontap.plugins.module_utils.rest_volume as rest_volume
 
 HAS_NETAPP_LIB = netapp_utils.has_netapp_lib()
 
@@ -284,6 +329,7 @@ class NetAppOntapLUN(object):
             size_unit=dict(default='gb',
                            choices=['bytes', 'b', 'kb', 'mb', 'gb', 'tb',
                                     'pb', 'eb', 'zb', 'yb'], type='str'),
+            comment=dict(required=False, type='str'),
             force_resize=dict(default=False, type='bool'),
             force_remove=dict(default=False, type='bool'),
             force_remove_fenced=dict(default=False, type='bool'),
@@ -291,6 +337,7 @@ class NetAppOntapLUN(object):
             vserver=dict(required=True, type='str'),
             os_type=dict(required=False, type='str', aliases=['ostype']),
             qos_policy_group=dict(required=False, type='str'),
+            qos_adaptive_policy_group=dict(required=False, type='str'),
             space_reserve=dict(required=False, type='bool', default=True),
             space_allocation=dict(required=False, type='bool', default=False),
             use_exact_size=dict(required=False, type='bool', default=True),
@@ -311,12 +358,14 @@ class NetAppOntapLUN(object):
                 total_size=dict(type='int'),
                 total_size_unit=dict(choices=['bytes', 'b', 'kb', 'mb', 'gb', 'tb',
                                               'pb', 'eb', 'zb', 'yb'], type='str'),
+                scope=dict(type='str', choices=['application', 'auto', 'lun'], default='auto'),
             ))
         ))
 
         self.module = AnsibleModule(
             argument_spec=self.argument_spec,
-            supports_check_mode=True
+            supports_check_mode=True,
+            mutually_exclusive=[('qos_policy_group', 'qos_adaptive_policy_group')]
         )
 
         # set up state variables
@@ -410,13 +459,17 @@ class NetAppOntapLUN(object):
             if value is not None:
                 return_value[bool_attr_map[attr]] = self.na_helper.get_value_for_bool(True, value)
         str_attr_map = {
+            'comment': 'comment',
+            'multiprotocol-type': 'os_type',
             'name': 'name',
             'path': 'path',
             'qos-policy-group': 'qos_policy_group',
-            'multiprotocol-type': 'os_type'
+            'qos-adaptive-policy-group': 'qos_adaptive_policy_group',
         }
         for attr in str_attr_map:
             value = lun.get_child_content(attr)
+            if value is None and attr in ('comment', 'qos-policy-group', 'qos-adaptive-policy-group'):
+                value = ''
             if value is not None:
                 return_value[str_attr_map[attr]] = value
 
@@ -514,7 +567,7 @@ class NetAppOntapLUN(object):
             action = 'create'
         for option in required_options:
             if self.parameters.get(option) is None:
-                self.module.fail_json(msg='Error: "%s" is required to %s a san application.' % (option, action))
+                self.module.fail_json(msg="Error: '%s' is required to %s a san application." % (option, action))
 
         application_component = dict(name=self.parameters['name'])
         if not modify:
@@ -525,15 +578,16 @@ class NetAppOntapLUN(object):
                 value = self.na_helper.safe_get(self.parameters, ['san_application_template', attr])
                 if value is not None:
                     application_component[attr] = value
-        for attr in ('os_type', 'qos_policy_group', 'total_size'):
+        for attr in ('os_type', 'qos_policy_group', 'qos_adaptive_policy_group', 'total_size'):
             if not modify or attr in modify:
                 value = self.na_helper.safe_get(self.parameters, [attr])
                 if value is not None:
-                    if attr == 'qos_policy_group':
+                    # only one of them can be present at most
+                    if attr in ('qos_policy_group', 'qos_adaptive_policy_group'):
                         attr = 'qos'
                         value = dict(policy=dict(name=value))
                     application_component[attr] = value
-        tiering = self.na_helper.safe_get(self.parameters, ['nas_application_template', 'tiering'])
+        tiering = self.na_helper.safe_get(self.parameters, ['san_application_template', 'tiering'])
         if tiering is not None and not modify:
             application_component['tiering'] = dict()
             for attr in ('control', 'policy', 'object_stores'):
@@ -587,6 +641,26 @@ class NetAppOntapLUN(object):
         dummy, error = self.rest_app.patch_application(body)
         self.fail_on_error(error)
 
+    def convert_to_san_application(self, scope):
+        '''First convert volume to smart container using POST
+           Second modify app to add new luns using PATCH
+        '''
+        # dummy modify, so that we don't fill in the body
+        modify = dict(dummy='dummy')
+        body, error = self.create_san_app_body(modify)
+        self.fail_on_error(error)
+        dummy, error = self.rest_app.create_application(body)
+        self.fail_on_error(error)
+        app_current, error = self.rest_app.get_application_uuid()
+        self.fail_on_error(error)
+        if app_current is None:
+            self.module.fail_json('Error: failed to create smart container for %s' % self.parameters['name'])
+        app_modify, app_modify_warning = self.app_changes(scope)
+        if app_modify_warning is not None:
+            self.warnings.append(app_modify_warning)
+        if app_modify:
+            self.modify_san_application(app_modify)
+
     def delete_san_application(self):
         '''Use REST application/applications san template to delete one or more LUNs'''
         dummy, error = self.rest_app.delete_application()
@@ -602,10 +676,14 @@ class NetAppOntapLUN(object):
                    'space-reservation-enabled': str(self.parameters['space_reserve']),
                    'space-allocation-enabled': str(self.parameters['space_allocation']),
                    'use-exact-size': str(self.parameters['use_exact_size'])}
+        if self.parameters.get('comment') is not None:
+            options['comment'] = self.parameters['comment']
         if self.parameters.get('os_type') is not None:
             options['ostype'] = self.parameters['os_type']
         if self.parameters.get('qos_policy_group') is not None:
             options['qos-policy-group'] = self.parameters['qos_policy_group']
+        if self.parameters.get('qos_adaptive_policy_group') is not None:
+            options['qos-adaptive-policy-group'] = self.parameters['qos_adaptive_policy_group']
         lun_create = netapp_utils.zapi.NaElement.create_node_with_children(
             'lun-create-by-size', **options)
 
@@ -662,7 +740,10 @@ class NetAppOntapLUN(object):
 
     def set_lun_value(self, path, key, value):
         key_to_zapi = dict(
+            comment=('lun-set-comment', 'comment'),
+            # The same ZAPI is used for both QOS attributes
             qos_policy_group=('lun-set-qos-policy-group', 'qos-policy-group'),
+            qos_adaptive_policy_group=('lun-set-qos-policy-group', 'qos-adaptive-policy-group'),
             space_allocation=('lun-set-space-alloc', 'enable'),
             space_reserve=('lun-set-space-reservation-info', 'enable')
         )
@@ -713,7 +794,7 @@ class NetAppOntapLUN(object):
         self.module.fail_json(**elements)
 
     def set_total_size(self, validate):
-        # fix total_size attribute, report error if total_size is misisng (or size is mssing)
+        # fix total_size attribute, report error if total_size is missing (or size is missing)
         attr = 'total_size'
         value = self.na_helper.safe_get(self.parameters, ['san_application_template', attr])
         if value is not None or not validate:
@@ -756,7 +837,7 @@ class NetAppOntapLUN(object):
                 self.warnings.append(warning)
                 modify.pop('total_size')
 
-    def app_changes(self):
+    def app_changes(self, scope):
         # find and validate app changes
         app_current, error = self.rest_app.get_application_details('san')
         self.fail_on_error(error)
@@ -776,8 +857,10 @@ class NetAppOntapLUN(object):
         # if component name does not match, assume a change at LUN level
         comp_name = app_current['name']
         if comp_name != self.parameters['name']:
-            return None, "name: %s does not match component name: %s" % (self.parameters['name'], comp_name)
-
+            msg = "desired component/volume name: %s does not match existing component name: %s" % (self.parameters['name'], comp_name)
+            if scope == 'application':
+                self.module.fail_json(msg='Error: ' + msg + ".  scope=%s" % scope)
+            return None, msg + ".  scope=%s, assuming 'lun' scope." % scope
         # restore app name
         app_current['name'] = app_name
 
@@ -811,20 +894,42 @@ class NetAppOntapLUN(object):
         app_modify_warning = None
         actions = list()
         if self.rest_app:
+            scope = self.na_helper.safe_get(self.parameters, ['san_application_template', 'scope'])
             app_current, error = self.rest_app.get_application_uuid()
             self.fail_on_error(error)
+            if scope == 'lun' and app_current is None:
+                self.module.fail_json('Application not found: %s.  scope=%s.' %
+                                      (self.na_helper.safe_get(self.parameters, ['san_application_template', 'name']), scope))
+        else:
+            # no application template, fall back to LUN only
+            scope = 'lun'
+
+        if self.rest_app and scope != 'lun':
             app_cd_action = self.na_helper.get_cd_action(app_current, self.parameters)
+            if app_cd_action == 'create':
+                # check if target volume already exists
+                cp_volume_name = self.parameters['name']
+                volume, error = rest_volume.get_volume(self.rest_api, self.parameters['vserver'], cp_volume_name)
+                self.fail_on_error(error)
+                if volume is not None:
+                    if scope == 'application':
+                        # volume already exists, but not as part of this application
+                        app_cd_action = 'convert'
+                    else:
+                        # default name already in use, ask user to clarify intent
+                        msg = "Error: volume '%s' already exists.  Please use a different group name, or use 'application' scope.  scope=%s"
+                        self.module.fail_json(msg=msg % (cp_volume_name, scope))
             if app_cd_action is not None:
                 actions.append('app_%s' % app_cd_action)
             if app_cd_action == 'create':
                 self.validate_app_create()
             if app_cd_action is None and app_current is not None:
-                app_modify, app_modify_warning = self.app_changes()
+                app_modify, app_modify_warning = self.app_changes(scope)
                 if app_modify:
                     actions.append('app_modify')
                     results['app_modify'] = dict(app_modify)
 
-        if app_cd_action is None and app_modify is None:
+        if app_cd_action is None and scope != 'application':
             # actions at LUN level
             lun_path, from_lun_path = None, None
             from_name = self.parameters.get('from_name')
@@ -855,7 +960,7 @@ class NetAppOntapLUN(object):
                     actions.append('lun_rename')
                     app_modify_warning = None       # reset warning as we found a match
             if lun_cd_action is not None:
-                actions.append(actions.append('lun_%s' % lun_cd_action))
+                actions.append('lun_%s' % lun_cd_action)
             if lun_cd_action is None and self.parameters['state'] == 'present':
                 # we already handled rename if required
                 current.pop('name', None)
@@ -866,25 +971,32 @@ class NetAppOntapLUN(object):
                     app_modify_warning = None       # reset warning as we found a match
             if lun_cd_action and self.rest_app and app_current:
                 msg = 'This module does not support %s a LUN by name %s a SAN application.' %\
-                      ('adding', 'to') if lun_cd_action == 'create' else ('removing', 'from')
-                self.warnings.append(msg)
+                    ('adding', 'to') if lun_cd_action == 'create' else ('removing', 'from')
+                if scope == 'auto':
+                    # ignore LUN not found, as name can be a group name
+                    self.warnings.append(msg + ".  scope=%s, assuming 'application'" % scope)
+                    if not app_modify:
+                        self.na_helper.changed = False
+                elif scope == 'lun':
+                    self.module.fail_json(msg=msg + ".  scope=%s." % scope)
                 lun_cd_action = None
-                self.na_helper.changed = False
             if lun_cd_action == 'create' and self.parameters.get('size') is None:
                 self.module.fail_json(msg="size is a required parameter for create.")
 
         if self.na_helper.changed and not self.module.check_mode:
             if app_cd_action == 'create':
                 self.create_san_application()
+            elif app_cd_action == 'convert':
+                self.convert_to_san_application(scope)
             elif app_cd_action == 'delete':
                 self.rest_app.delete_application()
-            elif app_modify:
-                self.modify_san_application(app_modify)
             elif lun_cd_action == 'create':
                 self.create_lun()
             elif lun_cd_action == 'delete':
                 self.delete_lun(lun_path)
             else:
+                if app_modify:
+                    self.modify_san_application(app_modify)
                 if lun_rename:
                     self.rename_lun(from_lun_path, lun_path)
                 size_changed = False
@@ -895,7 +1007,7 @@ class NetAppOntapLUN(object):
                     lun_modify.pop('size')
                 if lun_modify:
                     self.modify_lun(lun_path, lun_modify)
-                if not lun_modify and not lun_rename:
+                if not lun_modify and not lun_rename and not app_modify:
                     # size may not have changed
                     self.na_helper.changed = size_changed
 
