@@ -6,6 +6,9 @@
 from __future__ import absolute_import, division, print_function
 __metaclass__ = type
 
+'''
+na_ontap_quotas
+'''
 
 ANSIBLE_METADATA = {'metadata_version': '1.1',
                     'status': ['preview'],
@@ -41,7 +44,7 @@ options:
   quota_target:
     description:
     - The quota target of the type specified.
-    required: true
+    - Required to create or modify a rule.
     type: str
   qtree:
     description:
@@ -53,8 +56,8 @@ options:
   type:
     description:
     - The type of quota rule
+    - Required to create or modify a rule.
     choices: ['user', 'group', 'tree']
-    required: true
     type: str
   policy:
     description:
@@ -64,6 +67,12 @@ options:
     description:
     - Whether the specified volume should have quota status on or off.
     type: bool
+  perform_user_mapping:
+    description:
+    - Whether quota management will perform user mapping for the user specified in quota-target.
+    - User mapping can be specified only for a user quota rule.
+    type: bool
+    version_added: 20.12.0
   file_limit:
     description:
     - The number of files that the target can have.
@@ -84,6 +93,13 @@ options:
     description:
     - The amount of disk space the target would have to exceed before a message is logged.
     type: str
+  activate_quota_on_change:
+    description:
+    - Method to use to activate quota on a change.
+    choices: ['resize', 'reinitialize', 'none']
+    default: resize
+    type: str
+    version_added: 20.12.0
 '''
 
 EXAMPLES = """
@@ -101,6 +117,36 @@ EXAMPLES = """
         hostname: "{{ netapp_hostname }}"
         username: "{{ netapp_username }}"
         password: "{{ netapp_password }}"
+    - name: Resize quota
+      na_ontap_quotas:
+        state: present
+        vserver: ansible
+        volume: ansible
+        quota_target: /vol/ansible
+        type: user
+        policy: ansible
+        file_limit: 2
+        disk_limit: 3
+        set_quota_status: True
+        activate_quota_on_change: resize
+        hostname: "{{ netapp_hostname }}"
+        username: "{{ netapp_username }}"
+        password: "{{ netapp_password }}"
+    - name: Reinitialize quota
+      na_ontap_quotas:
+        state: present
+        vserver: ansible
+        volume: ansible
+        quota_target: /vol/ansible
+        type: user
+        policy: ansible
+        file_limit: 2
+        disk_limit: 3
+        set_quota_status: True
+        activate_quota_on_change: reinitialize
+        hostname: "{{ netapp_hostname }}"
+        username: "{{ netapp_username }}"
+        password: "{{ netapp_password }}"
     - name: modify quota
       na_ontap_quotas:
         state: present
@@ -133,7 +179,7 @@ RETURN = """
 
 """
 
-
+import time
 import traceback
 from ansible.module_utils.basic import AnsibleModule
 from ansible.module_utils._text import to_native
@@ -153,26 +199,42 @@ class NetAppONTAPQuotas(object):
             state=dict(required=False, choices=['present', 'absent'], default='present'),
             vserver=dict(required=True, type='str'),
             volume=dict(required=True, type='str'),
-            quota_target=dict(required=True, type='str'),
+            quota_target=dict(required=False, type='str'),
             qtree=dict(required=False, type='str', default=""),
-            type=dict(required=True, type='str', choices=['user', 'group', 'tree']),
+            type=dict(required=False, type='str', choices=['user', 'group', 'tree']),
             policy=dict(required=False, type='str'),
             set_quota_status=dict(required=False, type='bool'),
+            perform_user_mapping=dict(required=False, type='bool'),
             file_limit=dict(required=False, type='str'),
             disk_limit=dict(required=False, type='str'),
             soft_file_limit=dict(required=False, type='str'),
             soft_disk_limit=dict(required=False, type='str'),
-            threshold=dict(required=False, type='str')
+            threshold=dict(required=False, type='str'),
+            activate_quota_on_change=dict(required=False, type='str', choices=['resize', 'reinitialize', 'none'], default='resize')
         ))
 
         self.module = AnsibleModule(
             argument_spec=self.argument_spec,
+            required_by={
+                'policy': ['quota_target', 'type'],
+                'perform_user_mapping': ['quota_target', 'type'],
+                'file_limit': ['quota_target', 'type'],
+                'disk_limit': ['quota_target', 'type'],
+                'soft_file_limit': ['quota_target', 'type'],
+                'soft_disk_limit': ['quota_target', 'type'],
+                'threshold': ['quota_target', 'type'],
+            },
+            required_together=[['quota_target', 'type']],
             supports_check_mode=True
         )
 
         self.na_helper = NetAppModule()
         self.parameters = self.na_helper.set_parameters(self.module.params)
 
+        # converted blank parameter to * as shown in vsim
+        if self.parameters.get('quota_target') == "":
+            self.parameters['quota_target'] = '*'
+
         if HAS_NETAPP_LIB is False:
             self.module.fail_json(
                 msg="the python NetApp-Lib module is required")
@@ -205,6 +267,8 @@ class NetAppONTAPQuotas(object):
         Get quota details
         :return: name of volume if quota exists, None otherwise
         """
+        if self.parameters.get('type') is None:
+            return None
         quota_get = netapp_utils.zapi.NaElement('quota-list-entries-iter')
         query = {
             'query': {
@@ -225,13 +289,21 @@ class NetAppONTAPQuotas(object):
             self.module.fail_json(msg='Error fetching quotas info: %s' % to_native(error),
                                   exception=traceback.format_exc())
         if result.get_child_by_name('num-records') and int(result.get_child_content('num-records')) >= 1:
-            return_values = {'volume': result['attributes-list']['quota-entry']['volume'],
-                             'file_limit': result['attributes-list']['quota-entry']['file-limit'],
-                             'disk_limit': result['attributes-list']['quota-entry']['disk-limit'],
-                             'soft_file_limit': result['attributes-list']['quota-entry']['soft-file-limit'],
-                             'soft_disk_limit': result['attributes-list']['quota-entry']['soft-disk-limit'],
-                             'threshold': result['attributes-list']['quota-entry']['threshold']}
-            return return_values
+            # if quota-target is '*', the query treats it as a wildcard. But a blank entry is represented as '*'.
+            # Hence the need to loop through all records to find a match.
+            for quota_entry in result.get_child_by_name('attributes-list').get_children():
+                quota_target = quota_entry.get_child_content('quota-target')
+                if quota_target == self.parameters['quota_target']:
+                    return_values = {'volume': quota_entry.get_child_content('volume'),
+                                     'file_limit': quota_entry.get_child_content('file-limit'),
+                                     'disk_limit': quota_entry.get_child_content('disk-limit'),
+                                     'soft_file_limit': quota_entry.get_child_content('soft-file-limit'),
+                                     'soft_disk_limit': quota_entry.get_child_content('soft-disk-limit'),
+                                     'threshold': quota_entry.get_child_content('threshold')}
+                    value = self.na_helper.safe_get(quota_entry, ['perform-user-mapping'])
+                    if value is not None:
+                        return_values['perform_user_mapping'] = self.na_helper.get_value_for_bool(True, value)
+                    return return_values
         return None
 
     def quota_entry_set(self):
@@ -247,6 +319,8 @@ class NetAppONTAPQuotas(object):
             options['file-limit'] = self.parameters['file_limit']
         if self.parameters.get('disk_limit'):
             options['disk-limit'] = self.parameters['disk_limit']
+        if self.parameters.get('perform_user_mapping') is not None:
+            options['perform-user-mapping'] = str(self.parameters['perform_user_mapping'])
         if self.parameters.get('soft_file_limit'):
             options['soft-file-limit'] = self.parameters['soft_file_limit']
         if self.parameters.get('soft_disk_limit'):
@@ -296,6 +370,8 @@ class NetAppONTAPQuotas(object):
             options['file-limit'] = self.parameters['file_limit']
         if self.parameters.get('disk_limit'):
             options['disk-limit'] = self.parameters['disk_limit']
+        if self.parameters.get('perform_user_mapping') is not None:
+            options['perform-user-mapping'] = str(self.parameters['perform_user_mapping'])
         if self.parameters.get('soft_file_limit'):
             options['soft-file-limit'] = self.parameters['soft_file_limit']
         if self.parameters.get('soft_disk_limit'):
@@ -327,24 +403,45 @@ class NetAppONTAPQuotas(object):
                                   % (status, self.parameters['volume'], to_native(error)),
                                   exception=traceback.format_exc())
 
+    def resize_quota(self):
+        """
+        resize quota
+        """
+        quota = netapp_utils.zapi.NaElement.create_node_with_children(
+            'quota-resize', **{'volume': self.parameters['volume']})
+        try:
+            self.server.invoke_successfully(quota,
+                                            enable_tunneling=True)
+        except netapp_utils.zapi.NaApiError as error:
+            self.module.fail_json(msg='Error setting %s for %s: %s'
+                                  % ('quota-resize', self.parameters['volume'], to_native(error)),
+                                  exception=traceback.format_exc())
+
     def apply(self):
         """
         Apply action to quotas
         """
         netapp_utils.ems_log_event("na_ontap_quotas", self.server)
+        cd_action = None
         modify_quota_status = None
         modify_quota = None
+        quota_status = None
         current = self.get_quotas()
-        if 'set_quota_status' in self.parameters:
+        if self.parameters.get('type') is not None:
+            cd_action = self.na_helper.get_cd_action(current, self.parameters)
+            if cd_action is None:
+                modify_quota = self.na_helper.get_modified_attributes(current, self.parameters)
+        if 'set_quota_status' in self.parameters or modify_quota:
             quota_status = self.get_quota_status()
-            if quota_status is not None:
-                quota_status_action = self.na_helper.get_modified_attributes(
-                    {'set_quota_status': True if quota_status == 'on' else False}, self.parameters)
-                if quota_status_action:
-                    modify_quota_status = 'quota-on' if quota_status_action['set_quota_status'] else 'quota-off'
-        cd_action = self.na_helper.get_cd_action(current, self.parameters)
-        if cd_action is None:
-            modify_quota = self.na_helper.get_modified_attributes(current, self.parameters)
+        if 'set_quota_status' in self.parameters and quota_status is not None:
+            quota_status_action = self.na_helper.get_modified_attributes(
+                {'set_quota_status': True if quota_status == 'on' else False}, self.parameters)
+            if quota_status_action:
+                modify_quota_status = 'quota-on' if quota_status_action['set_quota_status'] else 'quota-off'
+        if modify_quota is not None and modify_quota_status is None and quota_status == 'on':
+            # do we need to resize or reinitialize:
+            if self.parameters['activate_quota_on_change'] in ['resize', 'reinitialize']:
+                modify_quota_status = self.parameters['activate_quota_on_change']
         if self.na_helper.changed:
             if self.module.check_mode:
                 pass
@@ -357,8 +454,15 @@ class NetAppONTAPQuotas(object):
                     for key in list(modify_quota):
                         modify_quota[key.replace("_", "-")] = modify_quota.pop(key)
                     self.quota_entry_modify(modify_quota)
-                if modify_quota_status is not None:
+                if modify_quota_status in ['quota-off', 'quota-on']:
                     self.on_or_off_quota(modify_quota_status)
+                elif modify_quota_status == 'resize':
+                    self.resize_quota()
+                elif modify_quota_status == 'reinitialize':
+                    self.on_or_off_quota('quota-off')
+                    time.sleep(10)  # status switch interval
+                    self.on_or_off_quota('quota-on')
+
         self.module.exit_json(changed=self.na_helper.changed)
 
 
