@@ -238,7 +238,7 @@ class TestMyModule(unittest.TestCase):
 
         with pytest.raises(AnsibleExitJson) as exc:
             my_obj.apply()
-        assert exc.value.args[0]['changed']
+        assert not exc.value.args[0]['changed']
 
     @patch('ansible_collections.netapp.ontap.plugins.module_utils.netapp.OntapRestAPI.send_request')
     def test_run_ontap_version_check_for_10_2_pass(self, mock_request):
@@ -251,7 +251,7 @@ class TestMyModule(unittest.TestCase):
 
         with pytest.raises(AnsibleExitJson) as exc:
             my_obj.apply()
-        assert exc.value.args[0]['changed']
+        assert not exc.value.args[0]['changed']
 
     @patch('ansible_collections.netapp.ontap.plugins.module_utils.netapp.OntapRestAPI.send_request')
     def test_run_ontap_version_check_for_9_2_fail(self, mock_request):
@@ -321,16 +321,18 @@ class TestMyModule(unittest.TestCase):
         my_obj = ontap_rest_info_module()
         gather_subset = ['application/applications', 'application/templates', 'cloud/targets', 'cluster/chassis', 'cluster/jobs',
                          'cluster/metrocluster/diagnostics', 'cluster/metrics', 'cluster/nodes', 'cluster/peers', 'cluster/schedules',
-                         'cluster/software', 'cluster/software/history', 'cluster/software/packages', 'name-services/dns', 'name-services/ldap',
-                         'name-services/name-mappings', 'name-services/nis', 'network/ethernet/broadcast-domains', 'network/ethernet/ports',
-                         'network/fc/logins', 'network/fc/wwpn-aliases', 'network/ip/interfaces', 'network/ip/routes',
-                         'network/ip/service-policies', 'network/ipspaces', 'protocols/cifs/services', 'protocols/cifs/shares',
+                         'cluster/software', 'cluster/software/download', 'cluster/software/history', 'cluster/software/packages',
+                         'name-services/dns', 'name-services/ldap', 'name-services/name-mappings', 'name-services/nis',
+                         'network/ethernet/broadcast-domains', 'network/ethernet/ports', 'network/fc/logins', 'network/fc/wwpn-aliases',
+                         'network/ip/interfaces', 'network/ip/routes', 'network/ip/service-policies', 'network/ipspaces',
+                         'protocols/cifs/home-directory/search-paths', 'protocols/cifs/services', 'protocols/cifs/shares',
                          'protocols/san/fcp/services', 'protocols/san/igroups', 'protocols/san/iscsi/credentials',
-                         'protocols/san/iscsi/services', 'protocols/san/lun-maps', 'storage/aggregates', 'storage/disks',
-                         'storage/flexcache/flexcaches', 'storage/flexcache/origins', 'storage/luns', 'storage/namespaces',
+                         'protocols/san/iscsi/services', 'protocols/san/lun-maps', 'security/accounts', 'security/roles', 'storage/aggregates',
+                         'storage/disks', 'storage/flexcache/flexcaches', 'storage/flexcache/origins', 'storage/luns', 'storage/namespaces',
                          'storage/ports', 'storage/qos/policies', 'storage/qtrees', 'storage/quota/reports', 'storage/quota/rules',
                          'storage/shelves', 'storage/snapshot-policies', 'storage/volumes', 'support/autosupport', 'support/autosupport/messages',
-                         'support/ems', 'support/ems/events', 'support/ems/filters', 'svm/peers', 'svm/peer-permissions', 'svm/svms']
+                         'support/ems', 'support/ems/destinations', 'support/ems/events', 'support/ems/filters', 'svm/peers', 'svm/peer-permissions',
+                         'svm/svms']
         mock_request.side_effect = [
             SRR['validate_ontap_version_pass'],
             SRR['get_subset_info'],
@@ -389,6 +391,11 @@ class TestMyModule(unittest.TestCase):
             SRR['get_subset_info'],
             SRR['get_subset_info'],
             SRR['get_subset_info'],
+            SRR['get_subset_info'],
+            SRR['get_subset_info'],
+            SRR['get_subset_info'],
+            SRR['get_subset_info'],
+            SRR['get_subset_info'],
         ]
 
         with pytest.raises(AnsibleExitJson) as exc:
@@ -404,16 +411,18 @@ class TestMyModule(unittest.TestCase):
         my_obj = ontap_rest_info_module()
         gather_subset = ['application/applications', 'application/templates', 'cloud/targets', 'cluster/chassis', 'cluster/jobs',
                          'cluster/metrocluster/diagnostics', 'cluster/metrics', 'cluster/nodes', 'cluster/peers', 'cluster/schedules',
-                         'cluster/software', 'cluster/software/history', 'cluster/software/packages', 'name-services/dns', 'name-services/ldap',
-                         'name-services/name-mappings', 'name-services/nis', 'network/ethernet/broadcast-domains', 'network/ethernet/ports',
-                         'network/fc/logins', 'network/fc/wwpn-aliases', 'network/ip/interfaces', 'network/ip/routes',
-                         'network/ip/service-policies', 'network/ipspaces', 'protocols/cifs/services', 'protocols/cifs/shares',
+                         'cluster/software', 'cluster/software/download', 'cluster/software/history', 'cluster/software/packages',
+                         'name-services/dns', 'name-services/ldap', 'name-services/name-mappings', 'name-services/nis',
+                         'network/ethernet/broadcast-domains', 'network/ethernet/ports', 'network/fc/logins', 'network/fc/wwpn-aliases',
+                         'network/ip/interfaces', 'network/ip/routes', 'network/ip/service-policies', 'network/ipspaces',
+                         'protocols/cifs/home-directory/search-paths', 'protocols/cifs/services', 'protocols/cifs/shares',
                          'protocols/san/fcp/services', 'protocols/san/igroups', 'protocols/san/iscsi/credentials',
-                         'protocols/san/iscsi/services', 'protocols/san/lun-maps', 'storage/aggregates', 'storage/disks',
-                         'storage/flexcache/flexcaches', 'storage/flexcache/origins', 'storage/luns', 'storage/namespaces',
+                         'protocols/san/iscsi/services', 'protocols/san/lun-maps', 'security/accounts', 'security/roles', 'storage/aggregates',
+                         'storage/disks', 'storage/flexcache/flexcaches', 'storage/flexcache/origins', 'storage/luns', 'storage/namespaces',
                          'storage/ports', 'storage/qos/policies', 'storage/qtrees', 'storage/quota/reports', 'storage/quota/rules',
                          'storage/shelves', 'storage/snapshot-policies', 'storage/volumes', 'support/autosupport', 'support/autosupport/messages',
-                         'support/ems', 'support/ems/events', 'support/ems/filters', 'svm/peers', 'svm/peer-permissions', 'svm/svms']
+                         'support/ems', 'support/ems/destinations', 'support/ems/events', 'support/ems/filters', 'svm/peers', 'svm/peer-permissions',
+                         'svm/svms']
         mock_request.side_effect = [
             SRR['validate_ontap_version_pass'],
             SRR['get_subset_info'],
@@ -472,6 +481,11 @@ class TestMyModule(unittest.TestCase):
             SRR['get_subset_info'],
             SRR['get_subset_info'],
             SRR['get_subset_info'],
+            SRR['get_subset_info'],
+            SRR['get_subset_info'],
+            SRR['get_subset_info'],
+            SRR['get_subset_info'],
+            SRR['get_subset_info'],
         ]
 
         with pytest.raises(AnsibleExitJson) as exc:
