@@ -7,12 +7,11 @@ from __future__ import (absolute_import, division, print_function)
 __metaclass__ = type
 import json
 import pytest
-import sys
 
-from ansible_collections.netapp.ontap.tests.unit.compat import unittest
-from ansible_collections.netapp.ontap.tests.unit.compat.mock import patch, Mock
 from ansible.module_utils import basic
 from ansible.module_utils._text import to_bytes
+from ansible_collections.netapp.ontap.tests.unit.compat import unittest
+from ansible_collections.netapp.ontap.tests.unit.compat.mock import patch, Mock
 import ansible_collections.netapp.ontap.plugins.module_utils.netapp as netapp_utils
 
 from ansible_collections.netapp.ontap.plugins.modules.na_ontap_volume \
@@ -30,12 +29,10 @@ def set_module_args(args):
 
 class AnsibleExitJson(Exception):
     """Exception class to be raised by module.exit_json and caught by the test case"""
-    pass
 
 
 class AnsibleFailJson(Exception):
     """Exception class to be raised by module.fail_json and caught by the test case"""
-    pass
 
 
 def exit_json(*args, **kwargs):  # pylint: disable=unused-argument
@@ -51,6 +48,16 @@ def fail_json(*args, **kwargs):  # pylint: disable=unused-argument
     raise AnsibleFailJson(kwargs)
 
 
+# REST API canned responses when mocking send_request
+SRR = {
+    # common responses
+    'is_rest': (200, {}, None),
+    'is_zapi': (400, {}, "Unreachable"),
+    'empty_good': (200, {}, None),
+    'end_of_sequence': (500, None, "Unexpected call to send_request"),
+}
+
+
 class MockONTAPConnection(object):
     ''' mock server connection to ONTAP host '''
 
@@ -66,6 +73,10 @@ class MockONTAPConnection(object):
         ''' mock invoke_successfully returning xml data '''
         self.xml_in = xml
         # print("request: ", xml.to_string())
+        request = xml.to_string().decode('utf-8')
+        print(request)
+        if request.startswith('<sis-get-iter>'):
+            return self.build_sis_info()
         if isinstance(self.kind, list):
             kind = self.kind.pop(0)
             if len(self.kind) == 0:
@@ -76,12 +87,16 @@ class MockONTAPConnection(object):
 
         if kind == 'volume':
             xml = self.build_volume_info(self.params)
+        if kind == 'flexgroup':
+            xml = self.build_flexgroup_info(self.params)
         elif kind == 'job_info':
             xml = self.build_job_info(self.job_error)
         elif kind == 'error_modify':
             xml = self.build_modify_error()
         elif kind == 'failure_modify_async':
             xml = self.build_failure_modify_async()
+        elif kind == 'missing_element_modify_async':
+            xml = self.build_missing_element_modify_async()
         elif kind == 'success_modify_async':
             xml = self.build_success_modify_async()
         elif kind == 'zapi_error':
@@ -101,6 +116,7 @@ class MockONTAPConnection(object):
                 'volume-attributes': {
                     'volume-id-attributes': {
                         'containing-aggregate-name': vol_details['aggregate'],
+                        'instance-uuid': 'uuid',
                         'junction-path': vol_details['junction_path'],
                         'style-extended': 'flexvol'
                     },
@@ -154,7 +170,7 @@ class MockONTAPConnection(object):
         return xml
 
     @staticmethod
-    def build_flex_group_info(vol_details):
+    def build_flexgroup_info(vol_details):
         ''' build xml data for flexGroup volume-attributes '''
         xml = netapp_utils.zapi.NaElement('xml')
         attributes = {
@@ -163,6 +179,7 @@ class MockONTAPConnection(object):
                 'volume-attributes': {
                     'volume-id-attributes': {
                         'aggr-list': vol_details['aggregate'],
+                        'flexgroup-uuid': 'uuid',
                         'junction-path': vol_details['junction_path'],
                         'style-extended': 'flexgroup'
                     },
@@ -176,7 +193,7 @@ class MockONTAPConnection(object):
                         'is-atime-update-enabled': 'true'
                     },
                     'volume-state-attributes': {
-                        'state': "online"
+                        'state': "online",
                     },
                     'volume-space-attributes': {
                         'space-guarantee': 'none',
@@ -189,6 +206,9 @@ class MockONTAPConnection(object):
                         'volume-security-unix-attributes': {
                             'permissions': vol_details['unix_permissions']
                         }
+                    },
+                    'volume-snapshot-autodelete-attributes': {
+                        'commitment': 'try'
                     }
                 }
             }
@@ -242,6 +262,18 @@ class MockONTAPConnection(object):
         xml.add_child_elem(attributes)
         return xml
 
+    @staticmethod
+    def build_missing_element_modify_async():
+        ''' build xml data for success modify async '''
+        xml = netapp_utils.zapi.NaElement('xml')
+        attributes = netapp_utils.zapi.NaElement('success-list')
+        info_list_obj = netapp_utils.zapi.NaElement('volume-modify-iter-info')      # no async!
+        info_list_obj.add_new_child('status', 'in_progress')
+        info_list_obj.add_new_child('jobid', '1234')
+        attributes.add_child_elem(info_list_obj)
+        xml.add_child_elem(attributes)
+        return xml
+
     @staticmethod
     def build_failure_modify_async():
         ''' build xml data for failure modify async '''
@@ -255,6 +287,21 @@ class MockONTAPConnection(object):
         xml.add_child_elem(attributes)
         return xml
 
+    @staticmethod
+    def build_sis_info():
+        ''' build xml data for sis config '''
+        xml = netapp_utils.zapi.NaElement('xml')
+        attributes = {
+            'num-records': 1,
+            'attributes-list': {
+                'sis-status-info': {
+                    'policy': 'testme'
+                }
+            }
+        }
+        xml.translate_struct(attributes)
+        return xml
+
 
 class TestMyModule(unittest.TestCase):
     ''' a group of related Unit Tests '''
@@ -326,7 +373,12 @@ class TestMyModule(unittest.TestCase):
         :param job_error: error message when getting job status.
         :return: na_ontap_volume object
         """
-        vol_obj = vol_module()
+        with patch('ansible_collections.netapp.ontap.plugins.module_utils.netapp.OntapRestAPI.send_request') as mock_send:
+            mock_send.side_effect = [
+                SRR['is_zapi'],
+                SRR['end_of_sequence']
+            ]
+            vol_obj = vol_module()
         vol_obj.ems_log_event = Mock(return_value=None)
         vol_obj.get_efficiency_policy = Mock(return_value='test_efficiency')
         vol_obj.volume_style = None
@@ -741,7 +793,8 @@ class TestMyModule(unittest.TestCase):
             'vserver': self.mock_vol['vserver'],
             'style_extended': 'flexgroup',
             'unix_permissions': '755',
-            'is_online': True
+            'is_online': True,
+            'uuid': 'uuid'
         }
         get_volume.side_effect = [
             current
@@ -766,7 +819,8 @@ class TestMyModule(unittest.TestCase):
             'vserver': self.mock_vol['vserver'],
             'style_extended': 'flexgroup',
             'size': 20971520,
-            'unix_permissions': '755'
+            'unix_permissions': '755',
+            'uuid': '1234'
         }
         get_volume.side_effect = [
             current
@@ -791,7 +845,8 @@ class TestMyModule(unittest.TestCase):
             'name': self.mock_vol['name'],
             'vserver': self.mock_vol['vserver'],
             'style_extended': 'flexgroup',
-            'unix_permissions': '777'
+            'unix_permissions': '777',
+            'uuid': '1234'
         }
         get_volume.side_effect = [
             current
@@ -819,7 +874,8 @@ class TestMyModule(unittest.TestCase):
             'name': self.mock_vol['name'],
             'vserver': self.mock_vol['vserver'],
             'style_extended': 'flexgroup',
-            'unix_permissions': '777'
+            'unix_permissions': '777',
+            'uuid': '1234'
         }
         get_volume.side_effect = [
             current
@@ -829,6 +885,32 @@ class TestMyModule(unittest.TestCase):
             obj.apply()
         assert exc.value.args[0]['changed']
 
+    @patch('ansible_collections.netapp.ontap.plugins.modules.na_ontap_volume.NetAppOntapVolume.get_volume')
+    def test_successful_modify_unix_permissions_flex_group_0_missing_result(self, get_volume):
+        ''' Test successful modify unix permissions flexGroup '''
+        data = self.mock_args('flexGroup_manual')
+        data['time_out'] = 0
+        data['unix_permissions'] = '---rw-r-xr-x'
+        set_module_args(data)
+        current = {
+            'hostname': 'test',
+            'username': 'test_user',
+            'password': 'test_pass!',
+            'name': self.mock_vol['name'],
+            'vserver': self.mock_vol['vserver'],
+            'style_extended': 'flexgroup',
+            'unix_permissions': '777',
+            'uuid': '1234'
+        }
+        get_volume.side_effect = [
+            current
+        ]
+        obj = self.get_volume_mock_object('missing_element_modify_async')
+        with pytest.raises(AnsibleFailJson) as exc:
+            obj.apply()
+        msg = "Unexpected error when modifying volume: result is:"
+        assert exc.value.args[0]['msg'].startswith(msg)
+
     @patch('ansible_collections.netapp.ontap.plugins.modules.na_ontap_volume.NetAppOntapVolume.check_job_status')
     @patch('ansible_collections.netapp.ontap.plugins.modules.na_ontap_volume.NetAppOntapVolume.get_volume')
     def test_error_modify_unix_permissions_flex_group(self, get_volume, check_job_status):
@@ -844,7 +926,8 @@ class TestMyModule(unittest.TestCase):
             'name': self.mock_vol['name'],
             'vserver': self.mock_vol['vserver'],
             'style_extended': 'flexgroup',
-            'unix_permissions': '777'
+            'unix_permissions': '777',
+            'uuid': '1234'
         }
         get_volume.side_effect = [
             current
@@ -868,8 +951,9 @@ class TestMyModule(unittest.TestCase):
             'password': 'test_pass!',
             'name': self.mock_vol['name'],
             'vserver': self.mock_vol['vserver'],
-            'style_extended': 'flexvol',
-            'unix_permissions': '777'
+            'style_extended': 'flexgroup',
+            'unix_permissions': '777',
+            'uuid': '1234'
         }
         get_volume.side_effect = [
             current
@@ -894,7 +978,8 @@ class TestMyModule(unittest.TestCase):
             'style_extended': 'flexgroup',
             'is_online': True,
             'junction_path': 'anything',
-            'unix_permissions': '755'
+            'unix_permissions': '755',
+            'uuid': '1234'
         }
         get_volume.side_effect = [
             current
@@ -918,7 +1003,8 @@ class TestMyModule(unittest.TestCase):
             'style_extended': 'flexgroup',
             'is_online': False,
             'junction_path': 'anything',
-            'unix_permissions': '755'
+            'unix_permissions': '755',
+            'uuid': '1234'
         }
         get_volume.side_effect = [
             current
@@ -961,22 +1047,22 @@ class TestMyModule(unittest.TestCase):
             obj.check_job_status('123')
         assert exc.value.args[0]['failed']
 
-    def test_error_assign_efficiency_policy(self):
+    def test_error_set_efficiency_policy(self):
         data = self.mock_args()
         data['efficiency_policy'] = 'test_policy'
         set_module_args(data)
         obj = self.get_volume_mock_object('zapi_error')
         with pytest.raises(AnsibleFailJson) as exc:
-            obj.assign_efficiency_policy()
+            obj.set_efficiency_config()
         assert exc.value.args[0]['msg'] == 'Error enable efficiency on volume test_vol: NetApp API failed. Reason - test:error'
 
-    def test_error_assign_efficiency_policy_async(self):
+    def test_error_set_efficiency_policy_async(self):
         data = self.mock_args()
         data['efficiency_policy'] = 'test_policy'
         set_module_args(data)
         obj = self.get_volume_mock_object('zapi_error')
         with pytest.raises(AnsibleFailJson) as exc:
-            obj.assign_efficiency_policy_async()
+            obj.set_efficiency_config_async()
         assert exc.value.args[0]['msg'] == 'Error enable efficiency on volume test_vol: NetApp API failed. Reason - test:error'
 
     def test_successful_modify_tiering_policy(self):
@@ -1035,7 +1121,8 @@ class TestMyModule(unittest.TestCase):
             'vserver': self.mock_vol['vserver'],
             'snapshot_auto_delete': {'delete_order': 'newest_first', 'destroy_list': 'lun_clone,vol_clone',
                                      'target_free_space': 30, 'prefix': 'test', 'commitment': 'try',
-                                     'state': 'on', 'trigger': 'snap_reserve', 'defer_delete': 'scheduled'}
+                                     'state': 'on', 'trigger': 'snap_reserve', 'defer_delete': 'scheduled'},
+            'uuid': '1234'
         }
         get_volume.side_effect = [
             current
@@ -1117,3 +1204,23 @@ class TestMyModule(unittest.TestCase):
         with pytest.raises(AnsibleFailJson) as exc:
             self.get_volume_mock_object('zapi_error').snapshot_restore_volume()
         assert exc.value.args[0]['msg'] == 'Error restoring volume test_vol: NetApp API failed. Reason - test:error'
+
+    def test_error_modify_flexvol_to_flexgroup(self):
+        ''' Test successful modify vserver_dr_protection '''
+        data = self.mock_args()
+        data['auto_provision_as'] = 'flexgroup'
+        set_module_args(data)
+        with pytest.raises(AnsibleFailJson) as exc:
+            self.get_volume_mock_object('volume').apply()
+        msg = 'Error: changing a volume from one backend to another is not allowed.  Current: flexvol, desired: flexgroup.'
+        assert msg == exc.value.args[0]['msg']
+
+    def test_error_modify_flexgroup_to_flexvol(self):
+        ''' Test successful modify vserver_dr_protection '''
+        data = self.mock_args()
+        data['aggregate_name'] = 'nothing'
+        set_module_args(data)
+        with pytest.raises(AnsibleFailJson) as exc:
+            self.get_volume_mock_object('flexgroup').apply()
+        msg = 'Error: aggregate_name option cannot be used with FlexGroups.'
+        assert msg == exc.value.args[0]['msg']
